/*
 * CoreFreq
 * Copyright (C) 2015-2025 CYRIL COURTIAT
 * Licenses: GPL2
 */

#define _GNU_SOURCE
#include <sys/types.h>
#include <stdio.h>
#include <string.h>
#include <locale.h>
#include <sched.h>
#include <errno.h>

#include "bitasm.h"
#include "corefreq-ui.h"
#include "corefreq-cli-rsc.h"
#include "corefreq-cli-rsc-en.h"
#include "corefreq-cli-rsc-fr.h"
#include "corefreq-cli-rsc-theme-dflt.h"
#include "corefreq-cli-rsc-theme-usr1.h"
#include "corefreq-cli-rsc-theme-usr2.h"

#define DEF_LDA(_rsc)							\
ATTRIBUTE _rsc##_##ATTR_ARRAY						\
	[THM_CNT]							\
	[(sizeof((ATTRIBUTE[]) _rsc##_##THM_DFLT_ATTR) / sizeof(ATTRIBUTE))] = \
{									\
	[THM_DFLT] = _rsc##_##THM_DFLT_ATTR,				\
	[THM_USR1] = _rsc##_##THM_USR1_ATTR,				\
	[THM_USR2] = _rsc##_##THM_USR2_ATTR				\
};									\
ASCII	_rsc##_##CODE_EN_ARRAY[] = _rsc##_##CODE_EN,			\
	_rsc##_##CODE_FR_ARRAY[] = _rsc##_##CODE_FR

#define DEF_LDB(_rsc)							\
ATTRIBUTE _rsc##_##ATTR_ARRAY						\
	[THM_CNT]							\
	[(sizeof((ATTRIBUTE[]) _rsc##_##THM_DFLT_ATTR) / sizeof(ATTRIBUTE))] = \
{									\
	[THM_DFLT] = _rsc##_##THM_DFLT_ATTR,				\
	[THM_USR1] = _rsc##_##THM_USR1_ATTR,				\
	[THM_USR2] = _rsc##_##THM_USR2_ATTR				\
}

#define LDV(attr_var, en_var, fr_var)					\
	.Attr = {							\
		[THM_DFLT] = attr_var[THM_DFLT],			\
		[THM_USR1] = attr_var[THM_USR1],			\
		[THM_USR2] = attr_var[THM_USR2] 			\
	},								\
	.Code = {							\
		[LOC_EN] = (ASCII*) en_var,				\
		[LOC_FR] = (ASCII*) fr_var				\
	}

#define LDA(_rsc)							\
[_rsc] = {								\
	LDV(_rsc##_ATTR_ARRAY, _rsc##_CODE_EN_ARRAY, _rsc##_CODE_FR_ARRAY),\
	.Size = {							\
		[LOC_EN] = sizeof(_rsc##_CODE_EN_ARRAY),		\
		[LOC_FR] = sizeof(_rsc##_CODE_FR_ARRAY)			\
	}								\
}

#define LDB(_rsc)							\
[_rsc] = {								\
	LDV(_rsc##_ATTR_ARRAY, hSpace, hSpace) ,			\
	.Size = {							\
		[LOC_EN] = sizeof(HSPACE),				\
		[LOC_FR] = sizeof(HSPACE)				\
	}								\
}

#define LDS(_rsc)							\
[_rsc] = {								\
	LDV(_rsc##_ATTR_ARRAY, _rsc##_CODE_EN, _rsc##_CODE_FR) ,	\
	.Size = {							\
		[LOC_EN] = __builtin_strlen(_rsc##_CODE_EN),		\
		[LOC_FR] = __builtin_strlen(_rsc##_CODE_FR)		\
	}								\
}

#define LDT(_rsc)							\
[_rsc] = {								\
	LDV(RSC_VOID_ATTR_ARRAY, _rsc##_CODE_EN, _rsc##_CODE_FR),	\
	.Size = {							\
		[LOC_EN] = __builtin_strlen(_rsc##_CODE_EN),		\
		[LOC_FR] = __builtin_strlen(_rsc##_CODE_FR)		\
	}								\
}

#define LDC(_rsc, _ref) 						\
[_rsc] = {								\
	LDV(_ref##_ATTR_ARRAY, _rsc##_CODE_EN, _rsc##_CODE_FR) ,	\
	.Size = {							\
		[LOC_EN] = __builtin_strlen(_rsc##_CODE_EN),		\
		[LOC_FR] = __builtin_strlen(_rsc##_CODE_FR)		\
	}								\
}

#define LDQ(_rsc)							\
[_rsc] = {								\
	LDV(RSC_VOID_ATTR_ARRAY, _rsc##_CODE, _rsc##_CODE),		\
	.Size = {							\
		[LOC_EN] = __builtin_strlen(_rsc##_CODE),		\
		[LOC_FR] = __builtin_strlen(_rsc##_CODE)		\
	}								\
}

DEF_LDB(RSC_UI);
DEF_LDB(RSC_VOID);

#ifndef NO_HEADER
DEF_LDA(RSC_LAYOUT_HEADER_PROC);
DEF_LDA(RSC_LAYOUT_HEADER_CPU);
DEF_LDA(RSC_LAYOUT_HEADER_ARCH);
DEF_LDA(RSC_LAYOUT_HEADER_CACHE_L1);
DEF_LDA(RSC_LAYOUT_HEADER_BCLK);
DEF_LDA(RSC_LAYOUT_HEADER_CACHES);
#endif /* NO_HEADER */

#ifndef NO_UPPER
DEF_LDA(RSC_LAYOUT_RULER_LOAD);
DEF_LDA(RSC_LAYOUT_RULER_REL_LOAD);
DEF_LDA(RSC_LAYOUT_RULER_ABS_LOAD);
#endif /* NO_UPPER */

#ifndef NO_LOWER
DEF_LDA(RSC_LAYOUT_MONITOR_FREQUENCY);
DEF_LDA(RSC_LAYOUT_MONITOR_INST);
DEF_LDA(RSC_LAYOUT_MONITOR_COMMON);
DEF_LDA(RSC_LAYOUT_MONITOR_TASKS);
DEF_LDA(RSC_LAYOUT_MONITOR_SLICE);
DEF_LDA(RSC_LAYOUT_CUSTOM_FIELD);
DEF_LDA(RSC_LAYOUT_RULER_FREQUENCY);
DEF_LDA(RSC_LAYOUT_RULER_FREQUENCY_AVG);
DEF_LDA(RSC_LAYOUT_RULER_FREQUENCY_PKG);
DEF_LDA(RSC_LAYOUT_RULER_INST);
DEF_LDA(RSC_LAYOUT_RULER_CYCLES);
DEF_LDA(RSC_LAYOUT_RULER_CSTATES);
DEF_LDA(RSC_LAYOUT_RULER_INTERRUPTS);
DEF_LDA(RSC_LAYOUT_RULER_PACKAGE);
DEF_LDA(RSC_LAYOUT_PACKAGE_PC);
DEF_LDA(RSC_LAYOUT_PACKAGE_PC02);
DEF_LDA(RSC_LAYOUT_PACKAGE_PC03);
DEF_LDA(RSC_LAYOUT_PACKAGE_PC04);
DEF_LDA(RSC_LAYOUT_PACKAGE_PC06);
DEF_LDA(RSC_LAYOUT_PACKAGE_PC07);
DEF_LDA(RSC_LAYOUT_PACKAGE_PC08);
DEF_LDA(RSC_LAYOUT_PACKAGE_PC09);
DEF_LDA(RSC_LAYOUT_PACKAGE_PC10);
DEF_LDA(RSC_LAYOUT_PACKAGE_MC06);
DEF_LDA(RSC_LAYOUT_PACKAGE_UNCORE);
DEF_LDA(RSC_LAYOUT_RULER_TASKS);
DEF_LDA(RSC_LAYOUT_TASKS_TRACKING);
DEF_LDA(RSC_LAYOUT_TASKS_STATE_SORTED);
DEF_LDA(RSC_LAYOUT_TASKS_RUNTIME_SORTED);
DEF_LDA(RSC_LAYOUT_TASKS_USRTIME_SORTED);
DEF_LDA(RSC_LAYOUT_TASKS_SYSTIME_SORTED);
DEF_LDA(RSC_LAYOUT_TASKS_PROCESS_SORTED);
DEF_LDA(RSC_LAYOUT_TASKS_COMMAND_SORTED);
DEF_LDA(RSC_LAYOUT_TASKS_REVERSE_SORT_OFF);
DEF_LDA(RSC_LAYOUT_TASKS_REVERSE_SORT_ON);
DEF_LDA(RSC_LAYOUT_TASKS_VALUE_SWITCH);
DEF_LDA(RSC_LAYOUT_TASKS_VALUE_OFF);
DEF_LDA(RSC_LAYOUT_TASKS_VALUE_ON);
DEF_LDA(RSC_LAYOUT_RULER_SENSORS);
DEF_LDA(RSC_LAYOUT_RULER_PWR_SOC);
DEF_LDA(RSC_LAYOUT_RULER_VOLTAGE);
DEF_LDA(RSC_LAYOUT_RULER_VPKG_SOC);
DEF_LDA(RSC_LAYOUT_RULER_ENERGY);
DEF_LDA(RSC_LAYOUT_RULER_POWER);
DEF_LDA(RSC_LAYOUT_RULER_SLICE);
DEF_LDA(RSC_LAYOUT_RULER_CUSTOM);
#endif /* NO_LOWER */

#ifndef NO_FOOTER
DEF_LDA(RSC_LAYOUT_FOOTER_TECH_TSC);
DEF_LDA(RSC_LAYOUT_FOOTER_VOLT_TEMP);
DEF_LDA(RSC_LAYOUT_FOOTER_SYSTEM);
#endif /* NO_FOOTER */

DEF_LDA(RSC_LAYOUT_CARD_CORE_ONLINE_COND0);
DEF_LDA(RSC_LAYOUT_CARD_CORE_ONLINE_COND1);
DEF_LDA(RSC_LAYOUT_CARD_CORE_OFFLINE);
DEF_LDA(RSC_LAYOUT_CARD_CLK);
DEF_LDA(RSC_LAYOUT_CARD_UNCORE);
DEF_LDA(RSC_LAYOUT_CARD_BUS);
DEF_LDA(RSC_LAYOUT_CARD_MC);
DEF_LDA(RSC_LAYOUT_CARD_LOAD);
DEF_LDA(RSC_LAYOUT_CARD_IDLE);
DEF_LDA(RSC_LAYOUT_CARD_RAM);
DEF_LDA(RSC_LAYOUT_CARD_TASK);

DEF_LDB(RSC_RUN_STATE_COLOR);
DEF_LDB(RSC_UNINT_STATE_COLOR);
DEF_LDB(RSC_ZOMBIE_STATE_COLOR);
DEF_LDB(RSC_SLEEP_STATE_COLOR);
DEF_LDB(RSC_WAIT_STATE_COLOR);
DEF_LDB(RSC_OTHER_STATE_COLOR);
DEF_LDB(RSC_TRACKER_STATE_COLOR);

DEF_LDB(RSC_SYSTEM_REGISTERS_COND0);
DEF_LDB(RSC_SYSTEM_REGISTERS_COND1);
DEF_LDB(RSC_SYSTEM_REGISTERS_COND2);
DEF_LDB(RSC_SYSTEM_REGISTERS_COND3);
DEF_LDB(RSC_SYSTEM_REGISTERS_COND4);

DEF_LDB(RSC_SYSINFO_PROC_COND0);
DEF_LDB(RSC_SYSINFO_PROC_COND1);
DEF_LDB(RSC_SYSINFO_PROC_COND2);
DEF_LDB(RSC_SYSINFO_PROC_COND3);

DEF_LDB(RSC_SYSINFO_ISA_COND_0_0);
DEF_LDB(RSC_SYSINFO_ISA_COND_0_1);
DEF_LDB(RSC_SYSINFO_ISA_COND_0_2);
DEF_LDB(RSC_SYSINFO_ISA_COND_0_3);
DEF_LDB(RSC_SYSINFO_ISA_COND_0_4);

DEF_LDB(RSC_SYSINFO_ISA_COND_1_0);
DEF_LDB(RSC_SYSINFO_ISA_COND_1_1);
DEF_LDB(RSC_SYSINFO_ISA_COND_1_2);
DEF_LDB(RSC_SYSINFO_ISA_COND_1_3);
DEF_LDB(RSC_SYSINFO_ISA_COND_1_4);

DEF_LDB(RSC_SYSINFO_FEATURES_COND0);
DEF_LDB(RSC_SYSINFO_FEATURES_COND1);
DEF_LDB(RSC_SYSINFO_FEATURES_COND2);
DEF_LDB(RSC_SYSINFO_FEATURES_COND3);
DEF_LDB(RSC_SYSINFO_FEATURES_COND4);

DEF_LDB(RSC_SYSINFO_TECH_COND0);
DEF_LDB(RSC_SYSINFO_TECH_COND1);

DEF_LDB(RSC_SYSINFO_PERFMON_COND0);
DEF_LDB(RSC_SYSINFO_PERFMON_COND1);
DEF_LDB(RSC_SYSINFO_PERFMON_COND2);
DEF_LDB(RSC_SYSINFO_PERFMON_COND3);
DEF_LDB(RSC_SYSINFO_PERFMON_COND4);

DEF_LDB(RSC_SYSINFO_PERFMON_HWP_CAP_COND0);
DEF_LDB(RSC_SYSINFO_PERFMON_HWP_CAP_COND1);

DEF_LDB(RSC_SYSINFO_PWR_THERMAL_COND0);
DEF_LDB(RSC_SYSINFO_PWR_THERMAL_COND1);
DEF_LDB(RSC_SYSINFO_PWR_THERMAL_COND2);
DEF_LDB(RSC_SYSINFO_PWR_THERMAL_COND3);
DEF_LDB(RSC_SYSINFO_PWR_THERMAL_COND4);
DEF_LDB(RSC_SYSINFO_PWR_THERMAL_COND5);
DEF_LDB(RSC_SYSINFO_PWR_THERMAL_COND6);

DEF_LDB(RSC_SYSINFO_KERNEL);

DEF_LDB(RSC_TOPOLOGY_COND0);
DEF_LDB(RSC_TOPOLOGY_COND1);
DEF_LDB(RSC_TOPOLOGY_COND2);
DEF_LDB(RSC_TOPOLOGY_COND3);
DEF_LDB(RSC_TOPOLOGY_COND4);

DEF_LDB(RSC_MEMORY_CONTROLLER_COND0);
DEF_LDB(RSC_MEMORY_CONTROLLER_COND1);

DEF_LDB(RSC_CREATE_MENU_DISABLE);
DEF_LDB(RSC_MENU_ITEM_MENU);
DEF_LDB(RSC_MENU_ITEM_VIEW);
DEF_LDB(RSC_MENU_ITEM_WINDOW);
DEF_LDB(RSC_CREATE_MENU_FN_KEY);
DEF_LDB(RSC_CREATE_MENU_SHORTKEY);
DEF_LDB(RSC_CREATE_MENU_CTRL_KEY);

DEF_LDB(RSC_CREATE_SETTINGS_COND0);
DEF_LDB(RSC_CREATE_SETTINGS_COND1);

DEF_LDB(RSC_CREATE_ADV_HELP_COND0);
DEF_LDB(RSC_CREATE_ADV_HELP_COND1);

DEF_LDB(RSC_CREATE_HOTPLUG_CPU_ENABLE);
DEF_LDB(RSC_CREATE_HOTPLUG_CPU_DISABLE);
DEF_LDB(RSC_CREATE_HOTPLUG_CPU_ONLINE);
DEF_LDB(RSC_CREATE_HOTPLUG_CPU_OFFLINE);

DEF_LDB(RSC_CREATE_RATIO_CLOCK_COND0);
DEF_LDB(RSC_CREATE_RATIO_CLOCK_COND1);
DEF_LDB(RSC_CREATE_RATIO_CLOCK_COND2);
DEF_LDB(RSC_CREATE_RATIO_CLOCK_COND3);
DEF_LDB(RSC_CREATE_RATIO_CLOCK_COND4);
DEF_LDB(RSC_CREATE_RATIO_CLOCK_COND5);
DEF_LDB(RSC_CREATE_RATIO_CLOCK_COND6);

DEF_LDB(RSC_CREATE_SELECT_CPU_COND0);
DEF_LDB(RSC_CREATE_SELECT_CPU_COND1);
DEF_LDB(RSC_CREATE_SELECT_CPU_COND2);

#ifndef NO_FOOTER
DEF_LDB(RSC_HOT_EVENT_COND0);
DEF_LDB(RSC_HOT_EVENT_COND1);
DEF_LDB(RSC_HOT_EVENT_COND2);
DEF_LDB(RSC_HOT_EVENT_COND3);
DEF_LDB(RSC_HOT_EVENT_COND4);
#endif

DEF_LDB(RSC_BOX_EVENT_COND00);
DEF_LDB(RSC_BOX_EVENT_COND01);
DEF_LDB(RSC_BOX_EVENT_COND10);
DEF_LDB(RSC_BOX_EVENT_COND11);
DEF_LDB(RSC_BOX_EVENT_BUTTON);

DEF_LDB(RSC_CREATE_RECORDER);
DEF_LDB(RSC_SMBIOS_ITEM);

DEF_LDB(RSC_CREATE_SELECT_FREQ_PKG);
DEF_LDB(RSC_CREATE_SELECT_FREQ_COND0);
DEF_LDB(RSC_CREATE_SELECT_FREQ_COND1);

RESOURCE_ST Resource[] = {
	LDB(RSC_UI),
	LDB(RSC_VOID),
	LDQ(RSC_THEME_DFLT),
	LDQ(RSC_THEME_USR1),
	LDQ(RSC_THEME_USR2),
	LDQ(RSC_LAYOUT_LCD_RESET),
#ifndef NO_HEADER
	LDA(RSC_LAYOUT_HEADER_PROC),
	LDA(RSC_LAYOUT_HEADER_CPU),
	LDA(RSC_LAYOUT_HEADER_ARCH),
	LDA(RSC_LAYOUT_HEADER_CACHE_L1),
	LDA(RSC_LAYOUT_HEADER_BCLK),
	LDA(RSC_LAYOUT_HEADER_CACHES),
#endif /* NO_HEADER */
#ifndef NO_UPPER
	LDA(RSC_LAYOUT_RULER_LOAD),
	LDA(RSC_LAYOUT_RULER_REL_LOAD),
	LDA(RSC_LAYOUT_RULER_ABS_LOAD),
#endif /* NO_UPPER */
#ifndef NO_LOWER
	LDA(RSC_LAYOUT_MONITOR_FREQUENCY),
	LDA(RSC_LAYOUT_MONITOR_INST),
	LDA(RSC_LAYOUT_MONITOR_COMMON),
	LDA(RSC_LAYOUT_MONITOR_TASKS),
	LDA(RSC_LAYOUT_MONITOR_SLICE),
	LDA(RSC_LAYOUT_CUSTOM_FIELD),
	LDA(RSC_LAYOUT_RULER_FREQUENCY),
	LDA(RSC_LAYOUT_RULER_FREQUENCY_AVG),
	LDA(RSC_LAYOUT_RULER_FREQUENCY_PKG),
	LDA(RSC_LAYOUT_RULER_INST),
	LDA(RSC_LAYOUT_RULER_CYCLES),
	LDA(RSC_LAYOUT_RULER_CSTATES),
	LDA(RSC_LAYOUT_RULER_INTERRUPTS),
	LDA(RSC_LAYOUT_RULER_PACKAGE),
	LDA(RSC_LAYOUT_PACKAGE_PC),
	LDA(RSC_LAYOUT_PACKAGE_PC02),
	LDA(RSC_LAYOUT_PACKAGE_PC03),
	LDA(RSC_LAYOUT_PACKAGE_PC04),
	LDA(RSC_LAYOUT_PACKAGE_PC06),
	LDA(RSC_LAYOUT_PACKAGE_PC07),
	LDA(RSC_LAYOUT_PACKAGE_PC08),
	LDA(RSC_LAYOUT_PACKAGE_PC09),
	LDA(RSC_LAYOUT_PACKAGE_PC10),
	LDA(RSC_LAYOUT_PACKAGE_MC06),
	LDA(RSC_LAYOUT_PACKAGE_UNCORE),
	LDA(RSC_LAYOUT_RULER_TASKS),
	LDA(RSC_LAYOUT_TASKS_TRACKING),
	LDA(RSC_LAYOUT_TASKS_STATE_SORTED),
	LDA(RSC_LAYOUT_TASKS_RUNTIME_SORTED),
	LDA(RSC_LAYOUT_TASKS_USRTIME_SORTED),
	LDA(RSC_LAYOUT_TASKS_SYSTIME_SORTED),
	LDA(RSC_LAYOUT_TASKS_PROCESS_SORTED),
	LDA(RSC_LAYOUT_TASKS_COMMAND_SORTED),
	LDA(RSC_LAYOUT_TASKS_REVERSE_SORT_OFF),
	LDA(RSC_LAYOUT_TASKS_REVERSE_SORT_ON),
	LDA(RSC_LAYOUT_TASKS_VALUE_SWITCH),
	LDA(RSC_LAYOUT_TASKS_VALUE_OFF),
	LDA(RSC_LAYOUT_TASKS_VALUE_ON),
	LDA(RSC_LAYOUT_RULER_SENSORS),
	LDA(RSC_LAYOUT_RULER_PWR_SOC),
	LDA(RSC_LAYOUT_RULER_VOLTAGE),
	LDA(RSC_LAYOUT_RULER_VPKG_SOC),
	LDA(RSC_LAYOUT_RULER_ENERGY),
	LDA(RSC_LAYOUT_RULER_POWER),
	LDA(RSC_LAYOUT_RULER_SLICE),
	LDA(RSC_LAYOUT_RULER_CUSTOM),
#endif /* NO_LOWER */
#ifndef NO_FOOTER
	LDA(RSC_LAYOUT_FOOTER_TECH_TSC),
	LDA(RSC_LAYOUT_FOOTER_VOLT_TEMP),
	LDA(RSC_LAYOUT_FOOTER_SYSTEM),
	LDQ(RSC_LAYOUT_FOOTER_TSC_NONE),
	LDQ(RSC_LAYOUT_FOOTER_TSC_VAR),
	LDQ(RSC_LAYOUT_FOOTER_TSC_INV),
#endif /* NO_FOOTER */
	LDA(RSC_LAYOUT_CARD_CORE_ONLINE_COND0),
	LDA(RSC_LAYOUT_CARD_CORE_ONLINE_COND1),
	LDA(RSC_LAYOUT_CARD_CORE_OFFLINE),
	LDA(RSC_LAYOUT_CARD_CLK),
	LDA(RSC_LAYOUT_CARD_UNCORE),
	LDA(RSC_LAYOUT_CARD_BUS),
	LDA(RSC_LAYOUT_CARD_MC),
	LDA(RSC_LAYOUT_CARD_LOAD),
	LDA(RSC_LAYOUT_CARD_IDLE),
	LDA(RSC_LAYOUT_CARD_RAM),
	LDA(RSC_LAYOUT_CARD_TASK),
/* ATTRIBUTE */
	LDB(RSC_RUN_STATE_COLOR),
	LDB(RSC_UNINT_STATE_COLOR),
	LDB(RSC_ZOMBIE_STATE_COLOR),
	LDB(RSC_SLEEP_STATE_COLOR),
	LDB(RSC_WAIT_STATE_COLOR),
	LDB(RSC_OTHER_STATE_COLOR),
#ifndef NO_LOWER
	LDB(RSC_TRACKER_STATE_COLOR),
#endif
	LDB(RSC_SYSTEM_REGISTERS_COND0),
	LDB(RSC_SYSTEM_REGISTERS_COND1),
	LDB(RSC_SYSTEM_REGISTERS_COND2),
	LDB(RSC_SYSTEM_REGISTERS_COND3),
	LDB(RSC_SYSTEM_REGISTERS_COND4),
	LDB(RSC_SYSINFO_PROC_COND0),
	LDB(RSC_SYSINFO_PROC_COND1),
	LDB(RSC_SYSINFO_PROC_COND2),
	LDB(RSC_SYSINFO_PROC_COND3),
	LDB(RSC_SYSINFO_ISA_COND_0_0),
	LDB(RSC_SYSINFO_ISA_COND_0_1),
	LDB(RSC_SYSINFO_ISA_COND_0_2),
	LDB(RSC_SYSINFO_ISA_COND_0_3),
	LDB(RSC_SYSINFO_ISA_COND_0_4),
	LDB(RSC_SYSINFO_ISA_COND_1_0),
	LDB(RSC_SYSINFO_ISA_COND_1_1),
	LDB(RSC_SYSINFO_ISA_COND_1_2),
	LDB(RSC_SYSINFO_ISA_COND_1_3),
	LDB(RSC_SYSINFO_ISA_COND_1_4),
	LDB(RSC_SYSINFO_FEATURES_COND0),
	LDB(RSC_SYSINFO_FEATURES_COND1),
	LDB(RSC_SYSINFO_FEATURES_COND2),
	LDB(RSC_SYSINFO_FEATURES_COND3),
	LDB(RSC_SYSINFO_FEATURES_COND4),
	LDB(RSC_SYSINFO_TECH_COND0),
	LDB(RSC_SYSINFO_TECH_COND1),
	LDB(RSC_SYSINFO_PERFMON_COND0),
	LDB(RSC_SYSINFO_PERFMON_COND1),
	LDB(RSC_SYSINFO_PERFMON_COND2),
	LDB(RSC_SYSINFO_PERFMON_COND3),
	LDB(RSC_SYSINFO_PERFMON_COND4),
	LDB(RSC_SYSINFO_PERFMON_HWP_CAP_COND0),
	LDB(RSC_SYSINFO_PERFMON_HWP_CAP_COND1),
	LDB(RSC_SYSINFO_PWR_THERMAL_COND0),
	LDB(RSC_SYSINFO_PWR_THERMAL_COND1),
	LDB(RSC_SYSINFO_PWR_THERMAL_COND2),
	LDB(RSC_SYSINFO_PWR_THERMAL_COND3),
	LDB(RSC_SYSINFO_PWR_THERMAL_COND4),
	LDB(RSC_SYSINFO_PWR_THERMAL_COND5),
	LDB(RSC_SYSINFO_PWR_THERMAL_COND6),
	LDB(RSC_SYSINFO_KERNEL),
	LDB(RSC_TOPOLOGY_COND0),
	LDB(RSC_TOPOLOGY_COND1),
	LDB(RSC_TOPOLOGY_COND2),
	LDB(RSC_TOPOLOGY_COND3),
	LDB(RSC_TOPOLOGY_COND4),
	LDB(RSC_MEMORY_CONTROLLER_COND0),
	LDB(RSC_MEMORY_CONTROLLER_COND1),
	LDB(RSC_CREATE_MENU_DISABLE),
	LDB(RSC_CREATE_MENU_FN_KEY),
	LDB(RSC_CREATE_MENU_SHORTKEY),
	LDB(RSC_CREATE_MENU_CTRL_KEY),
	LDS(RSC_CREATE_SETTINGS_COND0),
	LDS(RSC_CREATE_SETTINGS_COND1),
	LDS(RSC_CREATE_ADV_HELP_COND0),
	LDS(RSC_CREATE_ADV_HELP_COND1),
	LDQ(RSC_CREATE_HOTPLUG_CPU_TITLE),
	LDS(RSC_CREATE_HOTPLUG_CPU_ENABLE),
	LDS(RSC_CREATE_HOTPLUG_CPU_DISABLE),
	LDS(RSC_CREATE_HOTPLUG_CPU_ONLINE),
	LDS(RSC_CREATE_HOTPLUG_CPU_OFFLINE),
	LDB(RSC_CREATE_RATIO_CLOCK_COND0),
	LDB(RSC_CREATE_RATIO_CLOCK_COND1),
	LDB(RSC_CREATE_RATIO_CLOCK_COND2),
	LDB(RSC_CREATE_RATIO_CLOCK_COND3),
	LDB(RSC_CREATE_RATIO_CLOCK_COND4),
	LDB(RSC_CREATE_RATIO_CLOCK_COND5),
	LDB(RSC_CREATE_RATIO_CLOCK_COND6),
	LDB(RSC_CREATE_SELECT_CPU_COND0),
	LDB(RSC_CREATE_SELECT_CPU_COND1),
	LDB(RSC_CREATE_SELECT_CPU_COND2),
#ifndef NO_FOOTER
	LDB(RSC_HOT_EVENT_COND0),
	LDB(RSC_HOT_EVENT_COND1),
	LDB(RSC_HOT_EVENT_COND2),
	LDB(RSC_HOT_EVENT_COND3),
	LDB(RSC_HOT_EVENT_COND4),
#endif
	LDB(RSC_BOX_EVENT_COND00),
	LDB(RSC_BOX_EVENT_COND01),
	LDB(RSC_BOX_EVENT_COND10),
	LDB(RSC_BOX_EVENT_COND11),
	LDB(RSC_BOX_EVENT_BUTTON),
	LDB(RSC_CREATE_RECORDER),
	LDB(RSC_SMBIOS_ITEM),
	LDB(RSC_CREATE_SELECT_FREQ_PKG),
	LDB(RSC_CREATE_SELECT_FREQ_COND0),
	LDB(RSC_CREATE_SELECT_FREQ_COND1),
/* ASCII */
	LDQ(RSC_COREFREQ_TITLE),
	LDT(RSC_PROCESSOR_TITLE),
	LDT(RSC_PROCESSOR),
	LDT(RSC_ARCHITECTURE),
	LDT(RSC_VENDOR_ID),
	LDQ(RSC_HYBRID),
	LDT(RSC_REVISION),
	LDT(RSC_SIGNATURE),
	LDT(RSC_STEPPING),
	LDT(RSC_ONLINE_CPU),
	LDT(RSC_BASE_CLOCK),
	LDT(RSC_FREQUENCY),
	LDT(RSC_RATIO),
	LDT(RSC_FACTORY),
	LDT(RSC_PERFORMANCE),
	LDT(RSC_TARGET),
	LDT(RSC_UNLOCK),
	LDT(RSC_LOCK),
	LDT(RSC_ENABLE),
	LDT(RSC_DISABLE),
	LDT(RSC_CAPABILITIES),
	LDT(RSC_LOWEST),
	LDT(RSC_EFFICIENT),
	LDT(RSC_GUARANTEED),
	LDT(RSC_HIGHEST),
	LDT(RSC_RECORDER),
	LDT(RSC_STRESS),
	LDQ(RSC_SYSGATE),
	LDQ(RSC_FREQ_UNIT_MHZ),
	LDQ(RSC_PPIN),
	LDQ(RSC_OPP),
	LDQ(RSC_UNCORE),
	LDQ(RSC_TURBO),
	LDQ(RSC_CPPC),
	LDQ(RSC_MAX),
	LDQ(RSC_MIN),
	LDQ(RSC_TGT),
	LDQ(RSC_TBH),
	LDQ(RSC_TBO),
	LDQ(RSC_ACT),
	LDT(RSC_SCOPE_NONE),
	LDT(RSC_SCOPE_THREAD),
	LDT(RSC_SCOPE_CORE),
	LDT(RSC_SCOPE_PACKAGE),
	LDT(RSC_SYS_REGS_TITLE),
	LDQ(RSC_SYS_REGS_SPACE),
	LDQ(RSC_SYS_REGS_NA),
	LDQ(RSC_SYS_REGS_HDR_CPU),
	LDQ(RSC_SYS_REG_HDR_FLAGS),
	LDT(RSC_SYS_REG_PSTATE),
	LDT(RSC_SYS_REG_FLAG_N),
	LDT(RSC_SYS_REG_FLAG_Z),
	LDT(RSC_SYS_REG_FLAG_C),
	LDT(RSC_SYS_REG_FLAG_V),
	LDT(RSC_SYS_REG_FLAG_D),
	LDT(RSC_SYS_REG_FLAG_A),
	LDT(RSC_SYS_REG_FLAG_I),
	LDT(RSC_SYS_REG_FLAG_F),
	LDT(RSC_SYS_REG_FLAG_EL),
	LDT(RSC_SYS_REG_FLAG_SM),
	LDT(RSC_SYS_REG_FLAG_SSBS),
	LDT(RSC_SYS_REG_FLAG_NMI),
	LDT(RSC_SYS_REG_FLAG_PAN),
	LDT(RSC_SYS_REG_FLAG_UAO),
	LDT(RSC_SYS_REG_FLAG_DIT),
	LDT(RSC_SYS_REG_FLAG_TCO),
	LDT(RSC_SYS_REG_FLAG_PM),
	LDT(RSC_SYS_REG_HCR),
	LDQ(RSC_SYS_REG_HDR11_HCR),
	LDQ(RSC_SYS_REG_HDR12_HCR),
	LDQ(RSC_SYS_REG_HDR21_HCR),
	LDQ(RSC_SYS_REG_HDR31_HCR),
	LDQ(RSC_SYS_REG_HDR41_HCR),
	LDT(RSC_SYS_REG_TWEDEL),
	LDT(RSC_SYS_REG_TWEDEN),
	LDT(RSC_SYS_REG_TID5),
	LDT(RSC_SYS_REG_DCT),
	LDT(RSC_SYS_REG_TTLBOS),
	LDT(RSC_SYS_REG_TTLBIS),
	LDT(RSC_SYS_REG_ENSCXT),
	LDT(RSC_SYS_REG_TOCU),
	LDT(RSC_SYS_REG_AMVOFF),
	LDT(RSC_SYS_REG_TICAB),
	LDT(RSC_SYS_REG_TID4),
	LDT(RSC_SYS_REG_GPF),
	LDT(RSC_SYS_REG_FIEN),
	LDT(RSC_SYS_REG_FWB),
	LDT(RSC_SYS_REG_NV2),
	LDT(RSC_SYS_REG_AT),
	LDT(RSC_SYS_REG_NV1),
	LDT(RSC_SYS_REG_NV),
	LDT(RSC_SYS_REG_API),
	LDT(RSC_SYS_REG_APK),
	LDT(RSC_SYS_REG_MIOCNC),
	LDT(RSC_SYS_REG_TEA),
	LDT(RSC_SYS_REG_TERR),
	LDT(RSC_SYS_REG_TLOR),
	LDT(RSC_SYS_REG_E2H),
	LDT(RSC_SYS_REG_ID),
	LDT(RSC_SYS_REG_CD),
	LDT(RSC_SYS_REG_RW),
	LDT(RSC_SYS_REG_TRVM),
	LDT(RSC_SYS_REG_HCD),
	LDT(RSC_SYS_REG_TDZ),
	LDT(RSC_SYS_REG_TGE),
	LDT(RSC_SYS_REG_TVM),
	LDT(RSC_SYS_REG_TTLB),
	LDT(RSC_SYS_REG_TPU),
	LDT(RSC_SYS_REG_TPCP),
	LDT(RSC_SYS_REG_TSW),
	LDT(RSC_SYS_REG_TACR),
	LDT(RSC_SYS_REG_TSC),
	LDT(RSC_SYS_REG_TID3),
	LDT(RSC_SYS_REG_TID2),
	LDT(RSC_SYS_REG_TID1),
	LDT(RSC_SYS_REG_TID0),
	LDT(RSC_SYS_REG_TWE),
	LDT(RSC_SYS_REG_TWI),
	LDT(RSC_SYS_REG_DC),
	LDT(RSC_SYS_REG_BSU),
	LDT(RSC_SYS_REG_FB),
	LDT(RSC_SYS_REG_VSE),
	LDT(RSC_SYS_REG_VI),
	LDT(RSC_SYS_REG_VF),
	LDT(RSC_SYS_REG_AMO),
	LDT(RSC_SYS_REG_IMO),
	LDT(RSC_SYS_REG_FMO),
	LDT(RSC_SYS_REG_PTW),
	LDT(RSC_SYS_REG_SWIO),
	LDT(RSC_SYS_REG_VM),
	LDQ(RSC_SYS_REG_HDR11_SCTL),
	LDQ(RSC_SYS_REG_HDR12_SCTL),
	LDQ(RSC_SYS_REG_HDR21_SCTL),
	LDQ(RSC_SYS_REG_HDR22_SCTL),
	LDQ(RSC_SYS_REG_HDR31_SCTL),
	LDQ(RSC_SYS_REG_HDR41_SCTL),
	LDT(RSC_SYS_REG_SCTL),
	LDT(RSC_SYS_REG_TIDCP),
	LDT(RSC_SYS_REG_SPINT),
	LDT(RSC_SYS_REG_NMI),
	LDT(RSC_SYS_REG_EnTP2),
	LDT(RSC_SYS_REG_TCSO),
	LDT(RSC_SYS_REG_TCSO1),
	LDT(RSC_SYS_REG_TCSO0),
	LDT(RSC_SYS_REG_EPAN),
	LDT(RSC_SYS_REG_EnALS),
	LDT(RSC_SYS_REG_EnAS0),
	LDT(RSC_SYS_REG_EnASR),
	LDT(RSC_SYS_REG_TME),
	LDT(RSC_SYS_REG_TME1),
	LDT(RSC_SYS_REG_TME0),
	LDT(RSC_SYS_REG_TMT),
	LDT(RSC_SYS_REG_TMT1),
	LDT(RSC_SYS_REG_TMT0),
	LDT(RSC_SYS_REG_TWE_D),
	LDT(RSC_SYS_REG_TWE_C),
	LDT(RSC_SYS_REG_TWE_E),
	LDT(RSC_SYS_REG_DSSBS),
	LDT(RSC_SYS_REG_ATA),
	LDT(RSC_SYS_REG_ATA1),
	LDT(RSC_SYS_REG_ATA0),
	LDT(RSC_SYS_REG_TCF),
	LDT(RSC_SYS_REG_TCF1),
	LDT(RSC_SYS_REG_TCF0),
	LDT(RSC_SYS_REG_ITFSB),
	LDT(RSC_SYS_REG_BT),
	LDT(RSC_SYS_REG_BT1),
	LDT(RSC_SYS_REG_BT0),
	LDT(RSC_SYS_REG_EnFPM),
	LDT(RSC_SYS_REG_MSCEn),
	LDT(RSC_SYS_REG_CMOW),
	LDT(RSC_SYS_REG_EnIA),
	LDT(RSC_SYS_REG_EnIB),
	LDT(RSC_SYS_REG_LSM),
	LDT(RSC_SYS_REG_LSMA),
	LDT(RSC_SYS_REG_LSMD),
	LDT(RSC_SYS_REG_EnDA),
	LDT(RSC_SYS_REG_UCI),
	LDT(RSC_SYS_REG_EE),
	LDT(RSC_SYS_REG_E0E),
	LDT(RSC_SYS_REG_SPAN),
	LDT(RSC_SYS_REG_EIS),
	LDT(RSC_SYS_REG_IESB),
	LDT(RSC_SYS_REG_TSCXT),
	LDT(RSC_SYS_REG_WXN),
	LDT(RSC_SYS_REG_nTWE),
	LDT(RSC_SYS_REG_nTWI),
	LDT(RSC_SYS_REG_UCT),
	LDT(RSC_SYS_REG_DZE),
	LDT(RSC_SYS_REG_EnDB),
	LDT(RSC_SYS_REG_I),
	LDT(RSC_SYS_REG_EOS),
	LDT(RSC_SYS_REG_RCTX),
	LDT(RSC_SYS_REG_UMA),
	LDT(RSC_SYS_REG_SED),
	LDT(RSC_SYS_REG_ITD),
	LDT(RSC_SYS_REG_nAA),
	LDT(RSC_SYS_REG_CP15B),
	LDT(RSC_SYS_REG_SA0),
	LDT(RSC_SYS_REG_SA1),
	LDT(RSC_SYS_REG_C),
	LDT(RSC_SYS_REG_A),
	LDT(RSC_SYS_REG_M),
	LDQ(RSC_SYS_REG_HDR11_SCTL2),
	LDQ(RSC_SYS_REG_HDR12_SCTL2),
	LDT(RSC_SYS_REG_SCTL2),
	LDT(RSC_SYS_REG_CPTM),
	LDT(RSC_SYS_REG_CPTM0),
	LDT(RSC_SYS_REG_CPTM1),
	LDT(RSC_SYS_REG_CPTA),
	LDT(RSC_SYS_REG_CPTA0),
	LDT(RSC_SYS_REG_CPTA1),
	LDT(RSC_SYS_REG_PACM),
	LDT(RSC_SYS_REG_PACM0),
	LDT(RSC_SYS_REG_PACM1),
	LDT(RSC_SYS_REG_128SR),
	LDT(RSC_SYS_REG_EASE),
	LDT(RSC_SYS_REG_ANERR),
	LDT(RSC_SYS_REG_ADERR),
	LDT(RSC_SYS_REG_NMEA),
	LDQ(RSC_SYS_REG_HDR_FPSR),
	LDT(RSC_SYS_REG_FPSR),
	LDT(RSC_SYS_REG_FPSR_QC),
	LDT(RSC_SYS_REG_FPSR_IDC),
	LDT(RSC_SYS_REG_FPSR_IXC),
	LDT(RSC_SYS_REG_FPSR_UFC),
	LDT(RSC_SYS_REG_FPSR_OFC),
	LDT(RSC_SYS_REG_FPSR_DZC),
	LDT(RSC_SYS_REG_FPSR_IOC),
	LDQ(RSC_SYS_REG_HDR_FPCR),
	LDT(RSC_SYS_REG_FPCR),
	LDT(RSC_SYS_REG_FPCR_AHP),
	LDT(RSC_SYS_REG_FPCR_DN),
	LDT(RSC_SYS_REG_FPCR_FZ),
	LDT(RSC_SYS_REG_FPCR_RM),
	LDT(RSC_SYS_REG_FPCR_FZH),
	LDQ(RSC_SYS_REG_HDR11_EL),
	LDQ(RSC_SYS_REG_HDR12_EL),
	LDT(RSC_SYS_REG_EL),
	LDT(RSC_SYS_REG_EL_EXEC),
	LDT(RSC_SYS_REG_EL_SEC),
	LDQ(RSC_SYS_REG_HDR_SVCR),
	LDT(RSC_SYS_REG_SVCR),
	LDT(RSC_SYS_REG_SVCR_ZA),
	LDT(RSC_SYS_REG_SVCR_SM),
	LDQ(RSC_SYS_REG_HDR_CPACR),
	LDT(RSC_SYS_REG_CPACR),
	LDT(RSC_SYS_REG_ACR_TCP),
	LDT(RSC_SYS_REG_ACR_TAM),
	LDT(RSC_SYS_REG_ACR_POE),
	LDT(RSC_SYS_REG_ACR_TTA),
	LDT(RSC_SYS_REG_ACR_SME),
	LDT(RSC_SYS_REG_ACR_FP),
	LDT(RSC_SYS_REG_ACR_ZEN),
	LDT(RSC_SYS_REG_ACR_R8),
	LDT(RSC_SYS_REG_ACR_R0),
	LDT(RSC_ISA_TITLE),
	LDQ(RSC_ISA_AES),
	LDT(RSC_ISA_AES_COMM),
	LDQ(RSC_ISA_PMULL),
	LDQ(RSC_ISA_LSE),
	LDT(RSC_ISA_LSE_COMM),
	LDQ(RSC_ISA_LSE128),
	LDQ(RSC_ISA_LSFE),
	LDT(RSC_ISA_LSFE_COMM),
	LDQ(RSC_ISA_CRC32),
	LDT(RSC_ISA_CRC32_COMM),
	LDQ(RSC_ISA_DP),
	LDT(RSC_ISA_DP_COMM),
	LDQ(RSC_ISA_EPAC),
	LDT(RSC_ISA_EPAC_COMM),
	LDQ(RSC_ISA_FCMA),
	LDT(RSC_ISA_FCMA_COMM),
	LDQ(RSC_ISA_FAMINMAX),
	LDT(RSC_ISA_FAMINMAX_COMM),
	LDQ(RSC_ISA_FHM),
	LDT(RSC_ISA_FHM_COMM),
	LDQ(RSC_ISA_FP),
	LDT(RSC_ISA_FP_COMM),
	LDQ(RSC_ISA_FPAC),
	LDT(RSC_ISA_FPAC_COMM),
	LDQ(RSC_ISA_FPACCOMBINE),
	LDT(RSC_ISA_FPACCOMBINE_COMM),
	LDQ(RSC_ISA_JSCVT),
	LDT(RSC_ISA_JSCVT_COMM),
	LDQ(RSC_ISA_LRCPC),
	LDQ(RSC_ISA_LRCPC2),
	LDQ(RSC_ISA_LRCPC3),
	LDT(RSC_ISA_LRCPC_COMM),
	LDQ(RSC_ISA_LSUI),
	LDT(RSC_ISA_LSUI_COMM),
	LDQ(RSC_ISA_LUT),
	LDT(RSC_ISA_LUT_COMM),
	LDQ(RSC_ISA_MOPS),
	LDT(RSC_ISA_MOPS_COMM),
	LDQ(RSC_ISA_OCCMO),
	LDT(RSC_ISA_OCCMO_COMM),
	LDQ(RSC_ISA_PACGA),
	LDT(RSC_ISA_PACIMP_COMM),
	LDQ(RSC_ISA_PACQARMA3),
	LDT(RSC_ISA_PACQARMA3_COMM),
	LDQ(RSC_ISA_PACQARMA5),
	LDT(RSC_ISA_PACQARMA5_COMM),
	LDQ(RSC_ISA_PAUTH),
	LDT(RSC_ISA_PAUTH_COMM),
	LDQ(RSC_ISA_PAUTH2),
	LDT(RSC_ISA_PAUTH2_COMM),
	LDQ(RSC_ISA_PAUTH_LR),
	LDT(RSC_ISA_PAUTH_LR_COMM),
	LDQ(RSC_ISA_PACM),
	LDQ(RSC_ISA_PCDPHINT),
	LDT(RSC_ISA_PCDPHINT_COMM),
	LDQ(RSC_ISA_PRFMSLC),
	LDT(RSC_ISA_PRFMSLC_COMM),
	LDQ(RSC_ISA_FPRCVT),
	LDT(RSC_ISA_FPRCVT_COMM),
	LDQ(RSC_ISA_FRINTTS),
	LDT(RSC_ISA_FRINTTS_COMM),
	LDQ(RSC_ISA_SPECRES),
	LDT(RSC_ISA_SPECRES_COMM),
	LDQ(RSC_ISA_SPECRES2),
	LDQ(RSC_ISA_ATS1A),
	LDT(RSC_ISA_ATS1A_COMM),
	LDQ(RSC_ISA_BF16),
	LDT(RSC_ISA_BF16_COMM),
	LDQ(RSC_ISA_EBF16),
	LDT(RSC_ISA_EBF16_COMM),
	LDQ(RSC_ISA_CLRBHB),
	LDT(RSC_ISA_CLRBHB_COMM),
	LDQ(RSC_ISA_CONSTPACFLD),
	LDT(RSC_ISA_CONSTPACFLD_COMM),
	LDQ(RSC_ISA_CPA),
	LDT(RSC_ISA_CPA_COMM),
	LDQ(RSC_ISA_CSSC),
	LDT(RSC_ISA_CSSC_COMM),
	LDQ(RSC_ISA_HBC),
	LDT(RSC_ISA_HBC_COMM),
	LDQ(RSC_ISA_I8MM),
	LDT(RSC_ISA_I8MM_COMM),
	LDQ(RSC_ISA_RPRES),
	LDT(RSC_ISA_RPRES_COMM),
	LDQ(RSC_ISA_SB),
	LDT(RSC_ISA_SB_COMM),
	LDQ(RSC_ISA_SYSREG128),
	LDT(RSC_ISA_SYSREG128_COMM),
	LDQ(RSC_ISA_SYSINSTR128),
	LDT(RSC_ISA_SYSINSTR128_COMM),
	LDQ(RSC_ISA_TLBIW),
	LDT(RSC_ISA_TLBIW_COMM),
	LDQ(RSC_ISA_WFxT),
	LDT(RSC_ISA_WFxT_COMM),
	LDQ(RSC_ISA_XS),
	LDT(RSC_ISA_XS_COMM),
	LDQ(RSC_ISA_LS64),
	LDT(RSC_ISA_LS64_COMM),
	LDQ(RSC_ISA_LS64_V),
	LDQ(RSC_ISA_LS64_ACCDATA),
	LDQ(RSC_ISA_DGH),
	LDT(RSC_ISA_DGH_COMM),
	LDQ(RSC_ISA_DPB),
	LDT(RSC_ISA_DPB_COMM),
	LDQ(RSC_ISA_DPB2),
	LDQ(RSC_ISA_RAND),
	LDT(RSC_ISA_RAND_COMM),
	LDQ(RSC_ISA_RDMA),
	LDT(RSC_ISA_RDMA_COMM),
	LDQ(RSC_ISA_RNG_TRAP),
	LDT(RSC_ISA_RNG_TRAP_COMM),
	LDQ(RSC_ISA_RPRFM),
	LDT(RSC_ISA_RPRFM_COMM),
	LDQ(RSC_ISA_SHA1),
	LDQ(RSC_ISA_SHA256),
	LDQ(RSC_ISA_SHA512),
	LDQ(RSC_ISA_SHA3),
	LDT(RSC_ISA_SHA_COMM),
	LDQ(RSC_ISA_SM3),
	LDQ(RSC_ISA_SM4),
	LDT(RSC_ISA_SM_COMM),
	LDQ(RSC_ISA_SIMD),
	LDT(RSC_ISA_SIMD_COMM),
	LDQ(RSC_ISA_SME),
	LDT(RSC_ISA_SME_COMM),
	LDQ(RSC_ISA_SVE),
	LDT(RSC_ISA_SVE_COMM),
	LDQ(RSC_ISA_SVE_F64MM),
	LDQ(RSC_ISA_SVE_F32MM),
	LDQ(RSC_ISA_SVE_I8MM),
	LDQ(RSC_ISA_SVE_SM4),
	LDQ(RSC_ISA_SVE_SHA3),
	LDQ(RSC_ISA_SVE_BF16),
	LDQ(RSC_ISA_SVE_EBF16),
	LDQ(RSC_ISA_SVE_BitPerm),
	LDQ(RSC_ISA_SVE_AES),
	LDQ(RSC_ISA_SVE_PMULL128),
	LDQ(RSC_ISA_SVE2),
	LDQ(RSC_ISA_SME2),
	LDQ(RSC_ISA_SME2p1),
	LDQ(RSC_ISA_SME_FA64),
	LDQ(RSC_ISA_SME_LUTv2),
	LDQ(RSC_ISA_SME_I16I64),
	LDQ(RSC_ISA_SME_F64F64),
	LDQ(RSC_ISA_SME_I16I32),
	LDQ(RSC_ISA_SME_B16B16),
	LDQ(RSC_ISA_SME_F16F16),
	LDQ(RSC_ISA_SME_F8F16),
	LDQ(RSC_ISA_SME_F8F32),
	LDQ(RSC_ISA_SME_I8I32),
	LDQ(RSC_ISA_SME_F16F32),
	LDQ(RSC_ISA_SME_B16F32),
	LDQ(RSC_ISA_SME_BI32I32),
	LDQ(RSC_ISA_SME_F32F32),
	LDQ(RSC_ISA_SME_SF8FMA),
	LDQ(RSC_ISA_SME_SF8DP4),
	LDQ(RSC_ISA_SME_SF8DP2),
	LDQ(RSC_ISA_FlagM),
	LDT(RSC_ISA_FlagM_COMM),
	LDQ(RSC_ISA_FlagM2),
	LDQ(RSC_ISA_FP_ROUND),
	LDQ(RSC_ISA_FP_SH_VEC),
	LDQ(RSC_ISA_FP_SQRT),
	LDQ(RSC_ISA_FP_DIVIDE),
	LDQ(RSC_ISA_FP_TRAP),
	LDQ(RSC_ISA_FP_DP),
	LDQ(RSC_ISA_FP_SP),
	LDQ(RSC_ISA_FP_HP),
	LDQ(RSC_ISA_FP_NaN),
	LDQ(RSC_ISA_FP_FtZ),
	LDQ(RSC_ISA_FP_MISC),
	LDQ(RSC_ISA_SIMD_REG),
	LDQ(RSC_ISA_SIMD_FMA),
	LDQ(RSC_ISA_SIMD_HP),
	LDQ(RSC_ISA_SIMD_SP),
	LDQ(RSC_ISA_SIMD_INT),
	LDQ(RSC_ISA_SIMD_LS),
	LDQ(RSC_ISA_SIMD_MISC),
	LDT(RSC_FEATURES_TITLE),
	LDT(RSC_ON),
	LDT(RSC_OFF),
	LDT(RSC_FMW),
	LDT(RSC_NOT_AVAILABLE),
	LDT(RSC_AUTOMATIC),
	LDT(RSC_UNABLE),
	LDT(RSC_MISSING),
	LDT(RSC_PRESENT),
	LDT(RSC_VARIANT),
	LDT(RSC_INVARIANT),
	LDT(RSC_FEATURES_ACPI),
	LDT(RSC_FEATURES_AMU),
	LDT(RSC_FEATURES_BIG_END),
	LDT(RSC_FEATURES_BTI),
	LDT(RSC_FEATURES_EBEP),
	LDT(RSC_FEATURES_ECBHB),
	LDT(RSC_FEATURES_ECV),
	LDT(RSC_FEATURES_DF2),
	LDT(RSC_FEATURES_DIT),
	LDT(RSC_FEATURES_EXS),
	LDT(RSC_FEATURES_FGT),
	LDT(RSC_FEATURES_FPMR),
	LDT(RSC_FEATURES_PFAR),
	LDT(RSC_FEATURES_GCS),
	LDT(RSC_FEATURES_GIC),
	LDT(RSC_FEATURES_MPAM),
	LDT(RSC_FEATURES_MTE),
	LDT(RSC_FEATURES_MTE_FAR),
	LDT(RSC_FEATURES_MTE_PERM),
	LDT(RSC_FEATURES_MTE_STOREONLY),
	LDT(RSC_FEATURES_NMI),
	LDT(RSC_FEATURES_PA),
	LDT(RSC_FEATURES_PAN),
	LDT(RSC_FEATURES_RAS),
	LDT(RSC_FEATURES_RME),
	LDT(RSC_FEATURES_SEL2),
	LDT(RSC_FEATURES_THE),
	LDT(RSC_FEATURES_TLB),
	LDT(RSC_FEATURES_TME),
	LDT(RSC_FEATURES_TSC),
	LDT(RSC_FEATURES_UAO),
	LDT(RSC_FEATURES_UINJ),
	LDT(RSC_FEATURES_VA),
	LDT(RSC_FEATURES_VHE),
	LDT(RSC_FEAT_SECTION_MECH),
	LDT(RSC_FEAT_SECTION_SEC),
	LDT(RSC_TECHNOLOGIES_TITLE),
	LDT(RSC_TECHNOLOGIES_DCU),
	LDT(RSC_TECHNOLOGIES_MMU),
	LDT(RSC_TECHNOLOGIES_ICU),
	LDT(RSC_TECHNOLOGIES_VM),
	LDT(RSC_TECHNOLOGIES_VM_COMM),
	LDT(RSC_TECHNOLOGIES_IOMMU),
	LDT(RSC_TECHNOLOGIES_IOMMU_COMM),
	LDT(RSC_TECHNOLOGIES_SMT),
	LDT(RSC_TECHNOLOGIES_HYBRID),
	LDT(RSC_TECHNOLOGIES_HYPERV),
	LDQ(RSC_TECH_HYPERV_NONE),
	LDQ(RSC_TECH_BARE_METAL),
	LDQ(RSC_TECH_HYPERV_XEN),
	LDQ(RSC_TECH_HYPERV_KVM),
	LDQ(RSC_TECH_HYPERV_VBOX),
	LDQ(RSC_TECH_HYPERV_KBOX),
	LDQ(RSC_TECH_HYPERV_VMWARE),
	LDQ(RSC_TECH_HYPERV_HYPERV),
	LDT(RSC_PERF_MON_TITLE),
	LDT(RSC_PERF_CAPS_TITLE),
	LDT(RSC_VERSION),
	LDT(RSC_COUNTERS),
	LDT(RSC_GENERAL_CTRS),
	LDT(RSC_FIXED_CTRS),
	LDT(RSC_PERF_MON_UNIT_BIT),
	LDT(RSC_PERF_MON_CPPC),
	LDT(RSC_PERF_MON_PCT),
	LDT(RSC_PERF_MON_PSS),
	LDT(RSC_PERF_MON_PPC),
	LDT(RSC_PERF_MON_CPC),
	LDT(RSC_PERF_MON_CST),
	LDT(RSC_PERF_MON_HWP),
	LDT(RSC_PERF_MON_CORE_CSTATE),
	LDT(RSC_PERF_MON_CSTATE_BAR),
	LDT(RSC_PERF_MON_MONITOR_MWAIT),
	LDT(RSC_PERF_MON_MWAIT_IDX_CSTATE),
	LDT(RSC_PERF_MON_MWAIT_SUB_CSTATE),
	LDT(RSC_PERF_MON_CORE_CYCLE),
	LDT(RSC_PERF_MON_INST_RET),
	LDT(RSC_PERF_MON_PMC_COMM),
	LDT(RSC_PERF_MON_CPPC_COMM),
	LDQ(RSC_PERF_LABEL_VER),
	LDQ(RSC_PERF_LABEL_HWCF),
	LDQ(RSC_PERF_LABEL_CPPC),
	LDQ(RSC_PERF_LABEL_PCT),
	LDQ(RSC_PERF_LABEL_PSS),
	LDQ(RSC_PERF_LABEL_PPC),
	LDQ(RSC_PERF_LABEL_CPC),
	LDQ(RSC_PERF_LABEL_CST),
	LDQ(RSC_PERF_LABEL_HWP),
	LDQ(RSC_PERF_LABEL_CST_BAR),
	LDQ(RSC_PERF_LABEL_MWAIT_IDX),
	LDQ(RSC_PERF_ENCODING_C0),
	LDQ(RSC_PERF_ENCODING_C1),
	LDQ(RSC_PERF_ENCODING_C2),
	LDQ(RSC_PERF_ENCODING_C3),
	LDQ(RSC_PERF_ENCODING_C4),
	LDQ(RSC_PERF_ENCODING_C6),
	LDQ(RSC_PERF_ENCODING_C6R),
	LDQ(RSC_PERF_ENCODING_C7),
	LDQ(RSC_PERF_ENCODING_C7S),
	LDQ(RSC_PERF_ENCODING_C8),
	LDQ(RSC_PERF_ENCODING_C9),
	LDQ(RSC_PERF_ENCODING_C10),
	LDQ(RSC_PERF_ENCODING_UNS),
	LDT(RSC_POWER_THERMAL_TITLE),
	LDT(RSC_POWER_THERMAL_CPPC),
	LDT(RSC_POWER_THERMAL_TJMAX),
	LDT(RSC_POWER_THERMAL_DTS),
	LDT(RSC_POWER_THERMAL_PLN),
	LDT(RSC_POWER_THERMAL_PTM),
	LDT(RSC_POWER_THERMAL_UNITS),
	LDT(RSC_POWER_THERMAL_POWER),
	LDT(RSC_POWER_THERMAL_ENERGY),
	LDT(RSC_POWER_THERMAL_WINDOW),
	LDT(RSC_POWER_THERMAL_WATT),
	LDT(RSC_POWER_THERMAL_JOULE),
	LDT(RSC_POWER_THERMAL_SECOND),
	LDT(RSC_POWER_THERMAL_TDP),
	LDT(RSC_POWER_THERMAL_MIN),
	LDT(RSC_POWER_THERMAL_MAX),
	LDT(RSC_POWER_THERMAL_TPL),
	LDT(RSC_POWER_THERMAL_TW),
	LDT(RSC_POWER_THERMAL_EDC),
	LDT(RSC_POWER_THERMAL_TDC),
	LDT(RSC_POWER_THERMAL_POINT),
	LDT(RSC_THERMAL_POINT_THRESHOLD),
	LDT(RSC_THERMAL_POINT_LIMIT),
	LDT(RSC_THERMAL_POINT_THRESHOLD_1),
	LDT(RSC_THERMAL_POINT_THRESHOLD_2),
	LDT(RSC_THERMAL_POINT_TRIP_LIMIT),
	LDT(RSC_THERMAL_POINT_HTC_LIMIT),
	LDT(RSC_THERMAL_POINT_HTC_HYST),
	LDT(RSC_THERMAL_OFFSET_TITLE),
	LDQ(RSC_POWER_LABEL_CPPC),
	LDQ(RSC_POWER_LABEL_TJ),
	LDQ(RSC_POWER_LABEL_DTS),
	LDQ(RSC_POWER_LABEL_PLN),
	LDQ(RSC_POWER_LABEL_PTM),
	LDQ(RSC_POWER_LABEL_TDP),
	LDQ(RSC_POWER_LABEL_MIN),
	LDQ(RSC_POWER_LABEL_MAX),
	LDQ(RSC_POWER_LABEL_PL1),
	LDQ(RSC_POWER_LABEL_PL2),
	LDQ(RSC_POWER_LABEL_TW1),
	LDQ(RSC_POWER_LABEL_TW2),
	LDQ(RSC_POWER_LABEL_EDC),
	LDQ(RSC_POWER_LABEL_TDC),
	LDQ(RSC_POWER_LABEL_PKG),
	LDQ(RSC_POWER_LABEL_CORE),
	LDQ(RSC_POWER_LABEL_UNCORE),
	LDQ(RSC_POWER_LABEL_DRAM),
	LDQ(RSC_POWER_LABEL_PLATFORM),
	LDT(RSC_KERNEL_TITLE),
	LDC(RSC_KERNEL_TOTAL_RAM, RSC_SYSINFO_KERNEL),
	LDC(RSC_KERNEL_SHARED_RAM, RSC_SYSINFO_KERNEL),
	LDC(RSC_KERNEL_FREE_RAM, RSC_SYSINFO_KERNEL),
	LDC(RSC_KERNEL_BUFFER_RAM, RSC_SYSINFO_KERNEL),
	LDC(RSC_KERNEL_TOTAL_HIGH, RSC_SYSINFO_KERNEL),
	LDC(RSC_KERNEL_FREE_HIGH, RSC_SYSINFO_KERNEL),
	LDC(RSC_KERNEL_GOVERNOR, RSC_SYSINFO_KERNEL),
	LDC(RSC_KERNEL_FREQ_DRIVER, RSC_SYSINFO_KERNEL),
	LDC(RSC_KERNEL_IDLE_DRIVER, RSC_SYSINFO_KERNEL),
	LDC(RSC_KERNEL_CLOCK_SOURCE, RSC_SYSINFO_KERNEL),
	LDC(RSC_KERNEL_RELEASE, RSC_SYSINFO_KERNEL),
	LDC(RSC_KERNEL_VERSION, RSC_SYSINFO_KERNEL),
	LDC(RSC_KERNEL_MACHINE, RSC_SYSINFO_KERNEL),
	LDC(RSC_KERNEL_MEMORY, RSC_SYSINFO_KERNEL),
	LDC(RSC_KERNEL_STATE, RSC_SYSINFO_KERNEL),
	LDC(RSC_KERNEL_POWER, RSC_SYSINFO_KERNEL),
	LDC(RSC_KERNEL_LATENCY, RSC_SYSINFO_KERNEL),
	LDC(RSC_KERNEL_RESIDENCY, RSC_SYSINFO_KERNEL),
	LDC(RSC_KERNEL_LIMIT, RSC_SYSINFO_KERNEL),
	LDT(RSC_TOPOLOGY_TITLE),
	LDQ(RSC_TOPOLOGY_FMT0),
	LDQ(RSC_TOPOLOGY_FMT1),
	LDQ(RSC_TOPOLOGY_OFF_0),
	LDQ(RSC_TOPOLOGY_OFF_1),
	LDQ(RSC_TOPOLOGY_OFF_2),
	LDQ(RSC_TOPOLOGY_OFF_3),
	LDQ(RSC_TOPOLOGY_HDR_PKG),
	LDQ(RSC_TOPOLOGY_HDR_SMT),
	LDQ(RSC_TOPOLOGY_HDR_CACHE),
	LDQ(RSC_TOPOLOGY_HDR_WRBAK),
	LDQ(RSC_TOPOLOGY_HDR_INCL),
	LDQ(RSC_TOPOLOGY_HDR_EMPTY),
	LDQ(RSC_TOPOLOGY_SUB_ITEM1),
	LDQ(RSC_TOPOLOGY_SUB_ITEM3),
	LDQ(RSC_TOPOLOGY_SUB_ITEM4),
	LDQ(RSC_TOPOLOGY_SUB_ITEM5),
	LDQ(RSC_TOPOLOGY_SUB_ITEM6),
	LDQ(RSC_TOPOLOGY_ALT_ITEM1),
	LDQ(RSC_TOPOLOGY_ALT_ITEM2),
	LDQ(RSC_TOPOLOGY_ALT_ITEM3),
	LDQ(RSC_TOPOLOGY_ALT_ITEM4),
	LDQ(RSC_TOPOLOGY_BSP_COMM),
	LDT(RSC_MEM_CTRL_TITLE),
	LDQ(RSC_MEM_CTRL_UNIT_MHZ),
	LDQ(RSC_MEM_CTRL_UNIT_MTS),
	LDQ(RSC_MEM_CTRL_UNIT_MBS),
	LDQ(RSC_MEM_CTRL_MTY_CELL),
	LDQ(RSC_MEM_CTRL_CHANNEL),
	LDT(RSC_MEM_CTRL_SUBSECT1_0),
	LDT(RSC_MEM_CTRL_SUBSECT1_1),
	LDT(RSC_MEM_CTRL_SUBSECT1_2),
	LDT(RSC_MEM_CTRL_SINGLE_CHA_0),
	LDT(RSC_MEM_CTRL_SINGLE_CHA_1),
	LDT(RSC_MEM_CTRL_SINGLE_CHA_2),
	LDT(RSC_MEM_CTRL_DUAL_CHA_0),
	LDT(RSC_MEM_CTRL_DUAL_CHA_1),
	LDT(RSC_MEM_CTRL_DUAL_CHA_2),
	LDT(RSC_MEM_CTRL_TRIPLE_CHA_0),
	LDT(RSC_MEM_CTRL_TRIPLE_CHA_1),
	LDT(RSC_MEM_CTRL_TRIPLE_CHA_2),
	LDT(RSC_MEM_CTRL_QUAD_CHA_0),
	LDT(RSC_MEM_CTRL_QUAD_CHA_1),
	LDT(RSC_MEM_CTRL_QUAD_CHA_2),
	LDT(RSC_MEM_CTRL_SIX_CHA_0),
	LDT(RSC_MEM_CTRL_SIX_CHA_1),
	LDT(RSC_MEM_CTRL_SIX_CHA_2),
	LDT(RSC_MEM_CTRL_EIGHT_CHA_0),
	LDT(RSC_MEM_CTRL_EIGHT_CHA_1),
	LDT(RSC_MEM_CTRL_EIGHT_CHA_2),
	LDT(RSC_MEM_CTRL_TWELVE_CHA_0),
	LDT(RSC_MEM_CTRL_TWELVE_CHA_1),
	LDT(RSC_MEM_CTRL_TWELVE_CHA_2),
	LDT(RSC_MEM_CTRL_UNDEFINED_0),
	LDT(RSC_MEM_CTRL_UNDEFINED_1),
	LDT(RSC_MEM_CTRL_UNDEFINED_2),
	LDT(RSC_MEM_CTRL_BUS_RATE_0),
	LDT(RSC_MEM_CTRL_BUS_RATE_1),
	LDT(RSC_MEM_CTRL_BUS_SPEED_0),
	LDT(RSC_MEM_CTRL_BUS_SPEED_1),
	LDT(RSC_MEM_CTRL_RAM_STD_0),
	LDT(RSC_MEM_CTRL_RAM_STD_1),
	LDT(RSC_MEM_CTRL_RAM_STD_2),
	LDT(RSC_MEM_CTRL_RAM_STD_3),
	LDT(RSC_MEM_CTRL_DRAM_DDR2_0),
	LDT(RSC_MEM_CTRL_DRAM_DDR3_0),
	LDT(RSC_MEM_CTRL_DRAM_DDR4_0),
	LDT(RSC_MEM_CTRL_DRAM_DDR5_0),
	LDT(RSC_MEM_CTRL_DRAM_SPEED_0),
	LDT(RSC_MEM_CTRL_DRAM_SPEED_1),
	LDT(RSC_MEM_CTRL_SUBSECT2_0),
	LDT(RSC_MEM_CTRL_SUBSECT2_1),
	LDT(RSC_MEM_CTRL_SUBSECT2_2),
	LDT(RSC_MEM_CTRL_SUBSECT2_3),
	LDT(RSC_MEM_CTRL_SUBSECT2_4),
	LDT(RSC_MEM_CTRL_SUBSECT2_5),
	LDT(RSC_MEM_CTRL_DIMM_SLOT),
	LDT(RSC_MEM_CTRL_DIMM_BANK),
	LDT(RSC_MEM_CTRL_DIMM_RANK),
	LDT(RSC_MEM_CTRL_DIMM_ROW),
	LDT(RSC_MEM_CTRL_DIMM_COLUMN0),
	LDT(RSC_MEM_CTRL_DIMM_COLUMN1),
	LDT(RSC_MEM_CTRL_DIMM_SIZE_0),
	LDT(RSC_MEM_CTRL_DIMM_SIZE_1),
	LDT(RSC_MEM_CTRL_DIMM_SIZE_2),
	LDT(RSC_MEM_CTRL_DIMM_SIZE_3),
	LDQ(RSC_DDR3_CL),
	LDQ(RSC_DDR3_RCD),
	LDQ(RSC_DDR3_RP),
	LDQ(RSC_DDR3_RAS),
	LDQ(RSC_DDR3_RRD),
	LDQ(RSC_DDR3_RFC),
	LDQ(RSC_DDR3_WR),
	LDQ(RSC_DDR3_RTP),
	LDQ(RSC_DDR3_WTP),
	LDQ(RSC_DDR3_FAW),
	LDQ(RSC_DDR3_B2B),
	LDQ(RSC_DDR3_CWL),
	LDQ(RSC_DDR3_CMD),
	LDQ(RSC_DDR3_REFI),
	LDQ(RSC_DDR3_DDWRTRD),
	LDQ(RSC_DDR3_DRWRTRD),
	LDQ(RSC_DDR3_SRWRTRD),
	LDQ(RSC_DDR3_DDRDTWR),
	LDQ(RSC_DDR3_DRRDTWR),
	LDQ(RSC_DDR3_SRRDTWR),
	LDQ(RSC_DDR3_DDRDTRD),
	LDQ(RSC_DDR3_DRRDTRD),
	LDQ(RSC_DDR3_SRRDTRD),
	LDQ(RSC_DDR3_DDWRTWR),
	LDQ(RSC_DDR3_DRWRTWR),
	LDQ(RSC_DDR3_SRWRTWR),
	LDQ(RSC_DDR3_XS),
	LDQ(RSC_DDR3_XP),
	LDQ(RSC_DDR3_CKE),
	LDQ(RSC_DDR3_ECC),
	LDQ(RSC_DDR4_CL),
	LDQ(RSC_DDR4_RCD_R),
	LDQ(RSC_DDR4_RCD_W),
	LDQ(RSC_DDR4_RP),
	LDQ(RSC_DDR4_RAS),
	LDQ(RSC_DDR4_RRD),
	LDQ(RSC_DDR4_RFC),
	LDQ(RSC_DDR4_WR),
	LDQ(RSC_DDR4_RTP),
	LDQ(RSC_DDR4_WTP),
	LDQ(RSC_DDR4_FAW),
	LDQ(RSC_DDR4_GEAR),
	LDQ(RSC_DDR4_CWL),
	LDQ(RSC_DDR4_CMD),
	LDQ(RSC_DDR4_REFI),
	LDQ(RSC_DDR4_RDRD_SCL),
	LDQ(RSC_DDR4_RDRD_SC),
	LDQ(RSC_DDR4_RDRD_SD),
	LDQ(RSC_DDR4_RDRD_DD),
	LDQ(RSC_DDR4_RDWR_SCL),
	LDQ(RSC_DDR4_RDWR_SC),
	LDQ(RSC_DDR4_RDWR_SD),
	LDQ(RSC_DDR4_RDWR_DD),
	LDQ(RSC_DDR4_WRRD_SCL),
	LDQ(RSC_DDR4_WRRD_SC),
	LDQ(RSC_DDR4_WRRD_SD),
	LDQ(RSC_DDR4_WRRD_DD),
	LDQ(RSC_DDR4_WRWR_SCL),
	LDQ(RSC_DDR4_WRWR_SC),
	LDQ(RSC_DDR4_WRWR_SD),
	LDQ(RSC_DDR4_WRWR_DD),
	LDQ(RSC_DDR4_RRD_S),
	LDQ(RSC_DDR4_RRD_L),
	LDQ(RSC_DDR4_CKE),
	LDQ(RSC_DDR4_CPDED),
	LDQ(RSC_DDR4_ECC),
	LDQ(RSC_DDR4_ZEN_CL),
	LDQ(RSC_DDR4_ZEN_RP),
	LDQ(RSC_DDR4_ZEN_RAS),
	LDQ(RSC_DDR4_ZEN_RC),
	LDQ(RSC_DDR4_ZEN_FAW),
	LDQ(RSC_DDR4_ZEN_WTR_S),
	LDQ(RSC_DDR4_ZEN_WTR_L),
	LDQ(RSC_DDR4_ZEN_WR),
	LDQ(RSC_DDR4_ZEN_RDRD_SCL),
	LDQ(RSC_DDR4_ZEN_WRWR_SCL),
	LDQ(RSC_DDR4_ZEN_CWL),
	LDQ(RSC_DDR4_ZEN_RTP),
	LDQ(RSC_DDR4_ZEN_RDWR),
	LDQ(RSC_DDR4_ZEN_WRRD),
	LDQ(RSC_DDR4_ZEN_WRWR_SC),
	LDQ(RSC_DDR4_ZEN_WRWR_SD),
	LDQ(RSC_DDR4_ZEN_WRWR_DD),
	LDQ(RSC_DDR4_ZEN_RDRD_SC),
	LDQ(RSC_DDR4_ZEN_RDRD_SD),
	LDQ(RSC_DDR4_ZEN_RDRD_DD),
	LDQ(RSC_DDR4_ZEN_RTR_DLR),
	LDQ(RSC_DDR4_ZEN_WTW_DLR),
	LDQ(RSC_DDR4_ZEN_WTR_DLR),
	LDQ(RSC_DDR4_ZEN_RRD_DLR),
	LDQ(RSC_DDR4_ZEN_REFI),
	LDQ(RSC_DDR4_ZEN_RFC1),
	LDQ(RSC_DDR4_ZEN_RFC2),
	LDQ(RSC_DDR4_ZEN_RFC4),
	LDQ(RSC_DDR4_ZEN_RCPB),
	LDQ(RSC_DDR4_ZEN_RPPB),
	LDQ(RSC_DDR4_ZEN_BGS),
	LDQ(RSC_DDR4_ZEN_BGS_ALT),
	LDQ(RSC_DDR4_ZEN_BAN),
	LDQ(RSC_DDR4_ZEN_RCPAGE),
	LDQ(RSC_DDR4_ZEN_GDM),
	LDQ(RSC_DDR4_ZEN_ECC),
	LDQ(RSC_DDR4_ZEN_MRD),
	LDQ(RSC_DDR4_ZEN_MOD),
	LDQ(RSC_DDR4_ZEN_MRD_PDA),
	LDQ(RSC_DDR4_ZEN_MOD_PDA),
	LDQ(RSC_DDR4_ZEN_WRMPR),
	LDQ(RSC_DDR4_ZEN_STAG),
	LDQ(RSC_DDR4_ZEN_PDM),
	LDQ(RSC_DDR4_ZEN_RDDATA),
	LDQ(RSC_DDR4_ZEN_PHYWRD),
	LDQ(RSC_DDR4_ZEN_PHYWRL),
	LDQ(RSC_DDR4_ZEN_PHYRDL),
	LDQ(RSC_DDR5_ZEN_RFC_SB),
	LDQ(RSC_DDR5_ZEN_RCPB),
	LDQ(RSC_DDR5_ZEN_RPPB),
	LDQ(RSC_DDR5_ZEN_BGS),
	LDT(RSC_DDR3_CL_COMM),
	LDT(RSC_DDR3_RCD_COMM),
	LDT(RSC_DDR3_RP_COMM),
	LDT(RSC_DDR3_RAS_COMM),
	LDT(RSC_DDR3_RRD_COMM),
	LDT(RSC_DDR3_RFC_COMM),
	LDT(RSC_DDR3_WR_COMM),
	LDT(RSC_DDR3_RTP_COMM),
	LDT(RSC_DDR3_WTP_COMM),
	LDT(RSC_DDR3_FAW_COMM),
	LDT(RSC_DDR3_B2B_COMM),
	LDT(RSC_DDR3_CWL_COMM),
	LDT(RSC_DDR3_CMD_COMM),
	LDT(RSC_DDR3_REFI_COMM),
	LDT(RSC_DDR3_DDWRTRD_COMM),
	LDT(RSC_DDR3_DRWRTRD_COMM),
	LDT(RSC_DDR3_SRWRTRD_COMM),
	LDT(RSC_DDR3_DDRDTWR_COMM),
	LDT(RSC_DDR3_DRRDTWR_COMM),
	LDT(RSC_DDR3_SRRDTWR_COMM),
	LDT(RSC_DDR3_DDRDTRD_COMM),
	LDT(RSC_DDR3_DRRDTRD_COMM),
	LDT(RSC_DDR3_SRRDTRD_COMM),
	LDT(RSC_DDR3_DDWRTWR_COMM),
	LDT(RSC_DDR3_DRWRTWR_COMM),
	LDT(RSC_DDR3_SRWRTWR_COMM),
	LDT(RSC_DDR3_XS_COMM),
	LDT(RSC_DDR3_XP_COMM),
	LDT(RSC_DDR3_CKE_COMM),
	LDT(RSC_DDR3_ECC_COMM),
	LDT(RSC_DDR4_RCD_R_COMM),
	LDT(RSC_DDR4_RCD_W_COMM),
	LDT(RSC_DDR4_RDRD_SCL_COMM),
	LDT(RSC_DDR4_RDRD_SC_COMM),
	LDT(RSC_DDR4_RDRD_SD_COMM),
	LDT(RSC_DDR4_RDRD_DD_COMM),
	LDT(RSC_DDR4_RDWR_SCL_COMM),
	LDT(RSC_DDR4_RDWR_SC_COMM),
	LDT(RSC_DDR4_RDWR_SD_COMM),
	LDT(RSC_DDR4_RDWR_DD_COMM),
	LDT(RSC_DDR4_WRRD_SCL_COMM),
	LDT(RSC_DDR4_WRRD_SC_COMM),
	LDT(RSC_DDR4_WRRD_SD_COMM),
	LDT(RSC_DDR4_WRRD_DD_COMM),
	LDT(RSC_DDR4_WRWR_SCL_COMM),
	LDT(RSC_DDR4_WRWR_SC_COMM),
	LDT(RSC_DDR4_WRWR_SD_COMM),
	LDT(RSC_DDR4_WRWR_DD_COMM),
	LDT(RSC_DDR4_RRD_S_COMM),
	LDT(RSC_DDR4_RRD_L_COMM),
	LDT(RSC_DDR4_CPDED_COMM),
	LDT(RSC_DDR4_GEAR_COMM),
	LDT(RSC_DDR4_ZEN_RC_COMM),
	LDT(RSC_DDR4_ZEN_WTR_S_COMM),
	LDT(RSC_DDR4_ZEN_WTR_L_COMM),
	LDT(RSC_DDR4_ZEN_RDRD_SCL_COMM),
	LDT(RSC_DDR4_ZEN_WRWR_SCL_COMM),
	LDT(RSC_DDR4_ZEN_RTP_COMM),
	LDT(RSC_DDR4_ZEN_RDWR_COMM),
	LDT(RSC_DDR4_ZEN_WRRD_COMM),
	LDT(RSC_DDR4_ZEN_WRWR_SC_COMM),
	LDT(RSC_DDR4_ZEN_WRWR_SD_COMM),
	LDT(RSC_DDR4_ZEN_WRWR_DD_COMM),
	LDT(RSC_DDR4_ZEN_RDRD_SC_COMM),
	LDT(RSC_DDR4_ZEN_RDRD_SD_COMM),
	LDT(RSC_DDR4_ZEN_RDRD_DD_COMM),
	LDT(RSC_DDR4_ZEN_RTR_DLR_COMM),
	LDT(RSC_DDR4_ZEN_WTW_DLR_COMM),
	LDT(RSC_DDR4_ZEN_WTR_DLR_COMM),
	LDT(RSC_DDR4_ZEN_RRD_DLR_COMM),
	LDT(RSC_DDR4_ZEN_RFC1_COMM),
	LDT(RSC_DDR4_ZEN_RFC2_COMM),
	LDT(RSC_DDR4_ZEN_RFC4_COMM),
	LDT(RSC_DDR4_ZEN_RCPB_COMM),
	LDT(RSC_DDR4_ZEN_RPPB_COMM),
	LDT(RSC_DDR4_ZEN_BGS_COMM),
	LDT(RSC_DDR4_ZEN_BGS_ALT_COMM),
	LDT(RSC_DDR4_ZEN_BAN_COMM),
	LDT(RSC_DDR4_ZEN_RCPAGE_COMM),
	LDT(RSC_DDR4_ZEN_GDM_COMM),
	LDT(RSC_DDR4_ZEN_MRD_COMM),
	LDT(RSC_DDR4_ZEN_MOD_COMM),
	LDT(RSC_DDR4_ZEN_MRD_PDA_COMM),
	LDT(RSC_DDR4_ZEN_MOD_PDA_COMM),
	LDT(RSC_DDR4_ZEN_WRMPR_COMM),
	LDT(RSC_DDR4_ZEN_STAG_COMM),
	LDT(RSC_DDR4_ZEN_PDM_COMM),
	LDT(RSC_DDR4_ZEN_PHYWRD_COMM),
	LDT(RSC_DDR4_ZEN_PHYWRL_COMM),
	LDT(RSC_DDR4_ZEN_PHYRDL_COMM),
	LDT(RSC_DDR4_ZEN_RDDATA_COMM),
	LDT(RSC_DDR5_ZEN_RFC_SB_COMM),
	LDT(RSC_TASKS_SORTBY_STATE),
	LDT(RSC_TASKS_SORTBY_RTIME),
	LDT(RSC_TASKS_SORTBY_UTIME),
	LDT(RSC_TASKS_SORTBY_STIME),
	LDT(RSC_TASKS_SORTBY_PID),
	LDT(RSC_TASKS_SORTBY_COMM),
	LDS(RSC_MENU_ITEM_MENU),
	LDS(RSC_MENU_ITEM_VIEW),
	LDS(RSC_MENU_ITEM_WINDOW),
	LDT(RSC_MENU_ITEM_SPACER),
	LDT(RSC_MENU_ITEM_DATE_TIME),
	LDT(RSC_MENU_ITEM_FULL_TIME),
	LDT(RSC_MENU_ITEM_TINY_TIME),
	LDT(RSC_MENU_ITEM_SETTINGS),
	LDT(RSC_MENU_ITEM_SMBIOS),
	LDT(RSC_MENU_ITEM_KERNEL),
	LDT(RSC_MENU_ITEM_HOTPLUG),
	LDT(RSC_MENU_ITEM_TOOLS),
	LDT(RSC_MENU_ITEM_THEME),
	LDT(RSC_MENU_ITEM_ABOUT),
	LDT(RSC_MENU_ITEM_HELP),
	LDT(RSC_MENU_ITEM_KEYS),
	LDT(RSC_MENU_ITEM_LANG),
	LDT(RSC_MENU_ITEM_QUIT),
	LDT(RSC_MENU_ITEM_DASHBOARD),
	LDT(RSC_MENU_ITEM_FREQUENCY),
	LDT(RSC_MENU_ITEM_INST_CYCLES),
	LDT(RSC_MENU_ITEM_CORE_CYCLES),
	LDT(RSC_MENU_ITEM_IDLE_STATES),
	LDT(RSC_MENU_ITEM_PKG_CYCLES),
	LDT(RSC_MENU_ITEM_TASKS_MON),
	LDT(RSC_MENU_ITEM_SYS_INTER),
	LDT(RSC_MENU_ITEM_SENSORS),
	LDT(RSC_MENU_ITEM_VOLTAGE),
	LDT(RSC_MENU_ITEM_POWER),
	LDT(RSC_MENU_ITEM_SLICE_CTRS),
	LDT(RSC_MENU_ITEM_CUSTOM),
	LDT(RSC_MENU_ITEM_PROCESSOR),
	LDT(RSC_MENU_ITEM_TOPOLOGY),
	LDT(RSC_MENU_ITEM_FEATURES),
	LDT(RSC_MENU_ITEM_ISA_EXT),
	LDT(RSC_MENU_ITEM_TECH),
	LDT(RSC_MENU_ITEM_PERF_MON),
	LDT(RSC_MENU_ITEM_PERF_CAPS),
	LDT(RSC_MENU_ITEM_POW_THERM),
	LDT(RSC_MENU_ITEM_SYS_REGS),
	LDT(RSC_MENU_ITEM_MEM_CTRL),
	LDT(RSC_MENU_ITEM_EVENTS),
	LDT(RSC_SETTINGS_TITLE),
	LDT(RSC_SETTINGS_DAEMON),
	LDT(RSC_SETTINGS_INTERVAL),
	LDT(RSC_SETTINGS_SYS_TICK),
	LDT(RSC_SETTINGS_POLL_WAIT),
	LDT(RSC_SETTINGS_RING_WAIT),
	LDT(RSC_SETTINGS_CHILD_WAIT),
	LDT(RSC_SETTINGS_SLICE_WAIT),
	LDT(RSC_SETTINGS_RECORDER),
	LDT(RSC_SETTINGS_AUTO_CLOCK),
	LDT(RSC_SETTINGS_EXPERIMENTAL),
	LDT(RSC_SETTINGS_CPU_HOTPLUG),
	LDT(RSC_SETTINGS_PCI_ENABLED),
	LDT(RSC_SETTINGS_NMI_REGISTERED),
	LDT(RSC_SETTINGS_CPUIDLE_REGISTERED),
	LDT(RSC_SETTINGS_CPUFREQ_REGISTERED),
	LDT(RSC_SETTINGS_GOVERNOR_REGISTERED),
	LDT(RSC_SETTINGS_CS_REGISTERED),
	LDT(RSC_SETTINGS_THERMAL_SCOPE),
	LDT(RSC_SETTINGS_VOLTAGE_SCOPE),
	LDT(RSC_SETTINGS_POWER_SCOPE),
	LDT(RSC_SETTINGS_IDLE_ROUTE),
	LDT(RSC_SETTINGS_ROUTE_TITLE),
	LDQ(RSC_SETTINGS_ROUTE_DFLT),
	LDQ(RSC_SETTINGS_ROUTE_IO),
	LDQ(RSC_SETTINGS_ROUTE_HALT),
	LDQ(RSC_SETTINGS_ROUTE_MWAIT),
	LDT(RSC_HELP_TITLE),
	LDT(RSC_HELP_KEY_ESCAPE),
	LDT(RSC_HELP_KEY_SHIFT_TAB),
	LDT(RSC_HELP_KEY_TAB),
	LDT(RSC_HELP_KEY_UP),
	LDT(RSC_HELP_KEY_LEFT_RIGHT),
	LDT(RSC_HELP_KEY_DOWN),
	LDT(RSC_HELP_KEY_END),
	LDT(RSC_HELP_KEY_HOME),
	LDT(RSC_HELP_KEY_ENTER),
	LDT(RSC_HELP_KEY_PAGE_UP),
	LDT(RSC_HELP_KEY_PAGE_DOWN),
	LDT(RSC_HELP_KEY_MINUS),
	LDT(RSC_HELP_KEY_PLUS),
	LDQ(RSC_HELP_BLANK),
	LDT(RSC_HELP_KEY_MENU),
	LDT(RSC_HELP_MENU),
	LDT(RSC_HELP_CLOSE_WINDOW),
	LDT(RSC_HELP_PREV_WINDOW),
	LDT(RSC_HELP_NEXT_WINDOW),
	LDT(RSC_HELP_KEY_SHIFT_GR1),
	LDT(RSC_HELP_KEY_SHIFT_GR2),
	LDT(RSC_HELP_MOVE_WINDOW),
	LDT(RSC_HELP_KEY_ALT_GR3),
	LDT(RSC_HELP_SIZE_WINDOW),
	LDT(RSC_HELP_MOVE_SELECT),
	LDT(RSC_HELP_LAST_CELL),
	LDT(RSC_HELP_FIRST_CELL),
	LDT(RSC_HELP_TRIGGER_SELECT),
	LDT(RSC_HELP_PREV_PAGE),
	LDT(RSC_HELP_NEXT_PAGE),
	LDT(RSC_HELP_SCROLL_DOWN),
	LDT(RSC_HELP_SCROLL_UP),
	LDT(RSC_ADV_HELP_TITLE),
	LDT(RSC_ADV_HELP_SECT_FREQ),
	LDT(RSC_ADV_HELP_ITEM_AVG),
	LDT(RSC_ADV_HELP_SECT_TASK),
	LDT(RSC_ADV_HELP_ITEM_ORDER),
	LDT(RSC_ADV_HELP_ITEM_RST),
	LDT(RSC_ADV_HELP_ITEM_SEL),
	LDT(RSC_ADV_HELP_ITEM_REV),
	LDT(RSC_ADV_HELP_ITEM_HIDE),
	LDT(RSC_ADV_HELP_SECT_ANY),
	LDT(RSC_ADV_HELP_ITEM_POWER),
	LDT(RSC_ADV_HELP_ITEM_TOP),
	LDT(RSC_ADV_HELP_ITEM_UPD),
	LDT(RSC_ADV_HELP_ITEM_START),
	LDT(RSC_ADV_HELP_ITEM_STOP),
	LDT(RSC_ADV_HELP_ITEM_TOOLS),
	LDT(RSC_ADV_HELP_ITEM_GO_UP),
	LDT(RSC_ADV_HELP_ITEM_GO_DW),
	LDT(RSC_ADV_HELP_ITEM_TERMINAL),
	LDT(RSC_ADV_HELP_ITEM_PRT_SCR),
	LDT(RSC_ADV_HELP_ITEM_REC_SCR),
	LDT(RSC_ADV_HELP_ITEM_FAHR_CELS),
	LDT(RSC_ADV_HELP_ITEM_SYSGATE),
	LDT(RSC_ADV_HELP_ITEM_PROC_EVENT),
	LDT(RSC_ADV_HELP_ITEM_SECRET),
	LDT(RSC_TURBO_CLOCK_TITLE),
	LDT(RSC_RATIO_CLOCK_TITLE),
	LDT(RSC_UNCORE_CLOCK_TITLE),
	LDT(RSC_SELECT_CPU_TITLE),
	LDT(RSC_SELECT_FREQ_TITLE),
	LDT(RSC_BOX_DISABLE_COND0),
	LDT(RSC_BOX_DISABLE_COND1),
	LDT(RSC_BOX_ENABLE_COND0),
	LDT(RSC_BOX_ENABLE_COND1),
	LDT(RSC_BOX_INTERVAL_TITLE),
	LDQ(RSC_BOX_INTERVAL_STEP1),
	LDQ(RSC_BOX_INTERVAL_STEP2),
	LDQ(RSC_BOX_INTERVAL_STEP3),
	LDQ(RSC_BOX_INTERVAL_STEP4),
	LDQ(RSC_BOX_INTERVAL_STEP5),
	LDQ(RSC_BOX_INTERVAL_STEP6),
	LDQ(RSC_BOX_INTERVAL_STEP7),
	LDQ(RSC_BOX_INTERVAL_STEP8),
	LDQ(RSC_BOX_INTERVAL_STEP9),
	LDQ(RSC_BOX_INTERVAL_STEP10),
	LDT(RSC_BOX_AUTO_CLOCK_TITLE),
	LDT(RSC_BOX_MODE_TITLE),
	LDT(RSC_BOX_MODE_DESC),
	LDQ(RSC_BOX_CPPC_TITLE),
	LDQ(RSC_BOX_HWP_TITLE),
	LDT(RSC_BOX_HWP_DESC),
	LDT(RSC_BOX_FMW_DESC),
	LDQ(RSC_BOX_BLANK_DESC),
	LDT(RSC_BOX_NOMINAL_MODE_COND0),
	LDT(RSC_BOX_NOMINAL_MODE_COND1),
	LDT(RSC_BOX_EXPERIMENT_MODE_COND0),
	LDT(RSC_BOX_EXPERIMENT_MODE_COND1),
	LDT(RSC_BOX_INTERRUPT_TITLE),
	LDT(RSC_BOX_CPU_IDLE_TITLE),
	LDT(RSC_BOX_CPU_FREQ_TITLE),
	LDT(RSC_BOX_GOVERNOR_TITLE),
	LDT(RSC_BOX_CLOCK_SOURCE_TITLE),
	LDT(RSC_BOX_OPS_REGISTER_COND0),
	LDT(RSC_BOX_OPS_REGISTER_COND1),
	LDT(RSC_BOX_OPS_UNREGISTER_COND0),
	LDT(RSC_BOX_OPS_UNREGISTER_COND1),
	LDT(RSC_BOX_EVENT_TITLE),
	LDT(RSC_BOX_EVENT_SPACE),
	LDT(RSC_BOX_EVENT_THERMAL_SENSOR),
	LDT(RSC_BOX_EVENT_PROCHOT_STS),
	LDT(RSC_BOX_EVENT_CRITICAL_TEMP),
	LDT(RSC_BOX_EVENT_THOLD1_STS),
	LDT(RSC_BOX_EVENT_THOLD2_STS),
	LDT(RSC_BOX_EVENT_POWER_LIMIT),
	LDT(RSC_BOX_EVENT_CURRENT_LIMIT),
	LDT(RSC_BOX_EVENT_CROSS_DOM_LIMIT),
	LDT(RSC_BOX_EVENT_RESIDENCY),
	LDT(RSC_BOX_EVENT_AVG_THERMAL),
	LDT(RSC_BOX_EVENT_VR_THERMAL),
	LDT(RSC_BOX_EVENT_VR_TDC),
	LDT(RSC_BOX_EVENT_POWER_PL1),
	LDT(RSC_BOX_EVENT_POWER_PL2),
	LDT(RSC_BOX_EVENT_ELECTRICAL),
	LDT(RSC_BOX_EVENT_INEFFICIENCY),
	LDT(RSC_BOX_EVENT_MAX_TURBO),
	LDT(RSC_BOX_EVENT_TURBO_ATTEN),
	LDT(RSC_BOX_EVENT_THERMAL_TVB),
	LDT(RSC_BOX_EVENT_ALL_OF_THEM),
	LDT(RSC_BOX_POWER_POLICY_TITLE),
	LDT(RSC_BOX_POWER_POLICY_LOW),
	LDT(RSC_BOX_POWER_POLICY_HIGH),
	LDT(RSC_BOX_HWP_POLICY_MIN),
	LDT(RSC_BOX_HWP_POLICY_020),
	LDT(RSC_BOX_HWP_POLICY_040),
	LDT(RSC_BOX_HWP_POLICY_060),
	LDT(RSC_BOX_HWP_POLICY_MED),
	LDT(RSC_BOX_HWP_POLICY_0A0),
	LDT(RSC_BOX_HWP_POLICY_PWR),
	LDT(RSC_BOX_HWP_POLICY_0E0),
	LDT(RSC_BOX_HWP_POLICY_MAX),
	LDT(RSC_BOX_CFG_TDP_TITLE),
	LDT(RSC_BOX_CFG_TDP_DESC),
	LDQ(RSC_BOX_CFG_TDP_BLANK),
	LDT(RSC_BOX_CFG_TDP_LVL0),
	LDT(RSC_BOX_CFG_TDP_LVL1),
	LDT(RSC_BOX_CFG_TDP_LVL2),
	LDT(RSC_BOX_TDP_PKG_TITLE),
	LDT(RSC_BOX_TDP_CORES_TITLE),
	LDT(RSC_BOX_TDP_UNCORE_TITLE),
	LDT(RSC_BOX_TDP_RAM_TITLE),
	LDT(RSC_BOX_TDP_PLATFORM_TITLE),
	LDT(RSC_BOX_PL1_DESC),
	LDT(RSC_BOX_PL2_DESC),
	LDQ(RSC_BOX_PWR_OFFSET_00),
	LDQ(RSC_BOX_PWR_OFFSET_01),
	LDQ(RSC_BOX_PWR_OFFSET_02),
	LDQ(RSC_BOX_PWR_OFFSET_03),
	LDQ(RSC_BOX_PWR_OFFSET_04),
	LDQ(RSC_BOX_PWR_OFFSET_05),
	LDQ(RSC_BOX_PWR_OFFSET_06),
	LDQ(RSC_BOX_PWR_OFFSET_07),
	LDQ(RSC_BOX_PWR_OFFSET_08),
	LDQ(RSC_BOX_PWR_OFFSET_09),
	LDQ(RSC_BOX_PWR_OFFSET_10),
	LDQ(RSC_BOX_PWR_OFFSET_11),
	LDQ(RSC_BOX_PWR_OFFSET_12),
	LDQ(RSC_BOX_PWR_OFFSET_13),
	LDQ(RSC_BOX_CLAMPING_OFF_COND0),
	LDQ(RSC_BOX_CLAMPING_OFF_COND1),
	LDQ(RSC_BOX_CLAMPING_ON_COND0),
	LDQ(RSC_BOX_CLAMPING_ON_COND1),
	LDT(RSC_BOX_TDC_TITLE),
	LDT(RSC_BOX_TDC_DESC),
	LDQ(RSC_BOX_AMP_OFFSET_00),
	LDQ(RSC_BOX_AMP_OFFSET_01),
	LDQ(RSC_BOX_AMP_OFFSET_02),
	LDQ(RSC_BOX_AMP_OFFSET_03),
	LDQ(RSC_BOX_AMP_OFFSET_04),
	LDQ(RSC_BOX_AMP_OFFSET_05),
	LDQ(RSC_BOX_AMP_OFFSET_06),
	LDQ(RSC_BOX_AMP_OFFSET_07),
	LDQ(RSC_BOX_AMP_OFFSET_08),
	LDQ(RSC_BOX_AMP_OFFSET_09),
	LDQ(RSC_BOX_AMP_OFFSET_10),
	LDQ(RSC_BOX_AMP_OFFSET_11),
	LDQ(RSC_BOX_AMP_OFFSET_12),
	LDQ(RSC_BOX_AMP_OFFSET_13),
	LDT(RSC_BOX_TOOLS_TITLE),
	LDT(RSC_BOX_TOOLS_STOP_BURN),
	LDT(RSC_BOX_TOOLS_ATOMIC_BURN),
	LDT(RSC_BOX_TOOLS_CRC32_BURN),
	LDT(RSC_BOX_TOOLS_CONIC_BURN),
	LDT(RSC_BOX_TOOLS_RANDOM_CPU),
	LDT(RSC_BOX_TOOLS_ROUND_ROBIN_CPU),
	LDT(RSC_BOX_TOOLS_USER_CPU),
	LDT(RSC_BOX_TOOLS_MONTE_CARLO),
	LDT(RSC_BOX_CONIC_TITLE),
	LDT(RSC_BOX_CONIC_ITEM_1),
	LDT(RSC_BOX_CONIC_ITEM_2),
	LDT(RSC_BOX_CONIC_ITEM_3),
	LDT(RSC_BOX_CONIC_ITEM_4),
	LDT(RSC_BOX_CONIC_ITEM_5),
	LDT(RSC_BOX_CONIC_ITEM_6),
	LDT(RSC_BOX_LANG_TITLE),
	LDT(RSC_BOX_LANG_ENGLISH),
	LDT(RSC_BOX_LANG_FRENCH),
	LDQ(RSC_BOX_LANG_BLANK),
	LDT(RSC_BOX_THEME_TITLE),
	LDQ(RSC_BOX_THEME_BLANK),
	LDT(RSC_BOX_SCOPE_THERMAL_TITLE),
	LDT(RSC_BOX_SCOPE_VOLTAGE_TITLE),
	LDT(RSC_BOX_SCOPE_POWER_TITLE),
	LDT(RSC_BOX_SCOPE_NONE),
	LDT(RSC_BOX_SCOPE_THREAD),
	LDT(RSC_BOX_SCOPE_CORE),
	LDT(RSC_BOX_SCOPE_PACKAGE),
	LDT(RSC_ERROR_CMD_SYNTAX),
	LDT(RSC_ERROR_SHARED_MEM),
	LDT(RSC_ERROR_SYS_CALL),
	LDT(RSC_ERROR_UNIMPLEMENTED),
	LDT(RSC_ERROR_EXPERIMENTAL),
	LDT(RSC_ERROR_TURBO_PREREQ),
	LDT(RSC_ERROR_UNCORE_PREREQ),
	LDT(RSC_ERROR_PSTATE_NOT_FOUND),
	LDT(RSC_ERROR_CLOCKSOURCE),
	LDT(RSC_BOX_IDLE_LIMIT_TITLE),
	LDQ(RSC_BOX_IDLE_LIMIT_RESET),
	LDT(RSC_BOX_RECORDER_TITLE),
	LDT(RSC_SMBIOS_TITLE),
	LDT(RSC_MECH_CLRBHB),
	LDT(RSC_MECH_SSBD),
	LDT(RSC_MECH_SSBS),
	LDQ(RSC_CF0),
	LDQ(RSC_CF1),
	LDQ(RSC_CF2),
	LDQ(RSC_CF3),
	LDQ(RSC_CF4),
	LDQ(RSC_CF5),
	LDT(RSC_COPY0),
	LDT(RSC_COPY1),
	LDT(RSC_COPY2),
	LDT(RSC_CREATE_SELECT_AUTO_TURBO),
	LDT(RSC_CREATE_SELECT_FREQ_TURBO),
	LDT(RSC_CREATE_SELECT_FREQ_TGT),
	LDT(RSC_CREATE_SELECT_FREQ_HWP_TGT),
	LDT(RSC_CREATE_SELECT_FREQ_HWP_MAX),
	LDT(RSC_CREATE_SELECT_FREQ_HWP_MIN),
	LDT(RSC_CREATE_SELECT_FREQ_MAX),
	LDT(RSC_CREATE_SELECT_FREQ_MIN),
	LDT(RSC_CREATE_SELECT_FREQ_OFFLINE),
	LDT(RSC_POPUP_DRIVER_TITLE),
	LDT(RSC_EXIT_TITLE),
	LDT(RSC_EXIT_HEADER),
	LDT(RSC_EXIT_CONFIRM),
	LDT(RSC_EXIT_FOOTER)
};

#undef LDV
#undef LDA
#undef LDB
#undef LDS
#undef LDT
#undef LDC
#undef LDQ
#undef DEF_LDA
#undef DEF_LDB
